@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bf_lib.c - non-GUI general functions
  *
- * Copyright (C) 2000-2004 Olivier Sessink
+ * Copyright (C) 2000-20010 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,55 +18,150 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-/* #define DEBUG */
+/*#define DEBUG */
+
+#include "config.h"
+
+/* this is needed for Solaris to comply with the latest POSIX standard 
+ * regarding the ctime_r() function
+ */
+#ifdef PLATFORM_SOLARIS
+#define __EXTENSIONS__
+#define _POSIX_C_SOURCE 200312L
+#endif
 
 #include <gtk/gtk.h>
-#include <unistd.h> /* chdir() */
-#include <stdio.h> /* fopen() */
-#include <ctype.h> /* toupper */
-#include <string.h> /* strrchr strncmp memmove strncat*/
-#include <sys/stat.h> /* S_IFDIR */
-#include <errno.h> 	/* errno */
+#include <ctype.h>     /* toupper */
+#include <errno.h>     /* errno */
+#include <stdio.h>     /* fopen(), tempnam() */
+#include <string.h>    /* strrchr strncmp memmove strncat*/
+#include <sys/stat.h>  /* S_IFDIR */
+#include <time.h>      /* ctime_r() */
+#include <unistd.h>    /* chdir() */
 
 #include "bluefish.h"  /* for DEBUG_MSG and stuff like that */
-#include "bf_lib.h"  /* myself */
+#include "bf_lib.h"    /* myself */
 
-#ifdef WIN32
-#define DIRSTR "\\"
-#define DIRCHR 92
-#else
-#define DIRSTR "/"
-#define DIRCHR '/'
+/*#define REFP_REFS*/
+#ifdef REFP_REFS
+	gint num_refp_refs=0;
 #endif
-/**
- * get_filename_on_disk_encoding
- *
- * if gnome_vfs is defined, this function will also escape local paths
- * to make sure we can open files with a # in their name
- */
-gchar *get_filename_on_disk_encoding(const gchar *utf8filename) {
-	if (utf8filename) {
-		GError *gerror=NULL;
-		gsize b_written;
-		gchar *ondiskencoding = g_filename_from_utf8(utf8filename,-1, NULL,&b_written,&gerror);
-		if (gerror) {
-			g_print(_("Bluefish has trouble reading the filenames. Try to set the environment variable G_BROKEN_FILENAMES=1\n"));
-			ondiskencoding = g_strdup(utf8filename);
-		}
-#ifdef HAVE_GNOME_VFS
-		/* convert local path's */
-		if (ondiskencoding[0] == '/') {
-			gchar *tmp = gnome_vfs_escape_path_string(ondiskencoding);
-			g_free(ondiskencoding);
-			ondiskencoding = tmp;
-		}
-#endif /* HAVE_GNOME_VFS */
-		return ondiskencoding;
-	}
-	return NULL;
+
+#ifdef REFP_DEBUG
+void refcpointer_ref(Trefcpointer *rp) {
+	rp->count++;
+	g_print("refcpointer_ref, %p refcount=%d\n",rp, rp->count);
+}
+#endif
+Trefcpointer *refcpointer_new(gpointer data) {
+	Trefcpointer *rp = g_slice_new(Trefcpointer);
+#ifdef REFP_REFS
+	num_refp_refs++;
+	g_print("num_refp_refs=%d\n",num_refp_refs);
+#endif
+	rp->data = data;
+	rp->count = 1;
+#ifdef REFP_DEBUG
+	g_print("refcpointer_new, created %p with refcount 1\n",rp);
+#endif
+	return rp;
 }
 
-gchar *get_utf8filename_from_on_disk_encoding(const gchar *encodedname) {
+void refcpointer_unref(Trefcpointer *rp) {
+	rp->count--;
+#ifdef REFP_DEBUG
+	g_print("refcpointer_unref, %p refcount=%d %s\n",rp, rp->count, (rp->count ==0 ? "freeing data" : ""));
+#endif
+	if (rp->count <= 0) {
+#ifdef REFP_REFS
+		num_refp_refs--;
+		g_print("num_refp_refs=%d\n",num_refp_refs);
+#endif
+		g_free(rp->data);
+		g_slice_free(Trefcpointer,rp);
+	}
+}
+/*GFile *add_suffix_to_uri(GFile *file, const char *suffix) {
+	if (!suffix) {
+		g_object_ref(file);
+		return file;
+	} else {
+		gchar *tmp, *tmp2;
+		GFile *retval;
+		tmp = g_file_get_parse_name(file);
+		tmp2 =  g_strconcat(tmp,suffix,NULL);
+		retval = g_file_parse_name(tmp2);
+		g_free(tmp);
+		g_free(tmp2);
+		return retval;
+	}
+}*/
+GList *urilist_to_stringlist(GList *urilist) {
+	GList *retlist=NULL, *tmplist = g_list_last(urilist);
+	while (tmplist) {
+		retlist = g_list_prepend(retlist, g_file_get_parse_name((GFile *)tmplist->data));
+		tmplist = g_list_previous(tmplist);
+	}
+	return retlist;
+}
+
+void free_urilist(GList *urilist) {
+	GList *tmplist = g_list_first(urilist);
+	while (tmplist) {
+		g_object_unref((GObject *)tmplist->data);
+		tmplist = g_list_next(tmplist);
+	}
+	g_list_free(urilist);
+}
+
+#if GTK_CHECK_VERSION(2,14,0)
+
+#else
+GFile *gtk_file_chooser_get_file(GtkFileChooser *chooser) {
+	GFile *file=NULL;
+	gchar *uri = gtk_file_chooser_get_uri(chooser);
+	if (uri) {
+		file = g_file_new_for_uri(uri);
+		g_free(uri);
+	}
+	return file;
+}
+#endif
+
+GFile *user_bfdir(const gchar *filename) {
+	GFile *file;
+	gchar *path = g_strconcat(g_get_home_dir(), "/."PACKAGE"/", filename, NULL);
+	file = g_file_new_for_path(path);
+	g_free(path);
+	return file;
+}
+/*const gchar *full_path_utf8_from_uri(GFile *uri) {
+
+	GFileInfo* ginfo;
+	GError *error=NULL;
+	const gchar *utf8uri; 
+	ginfo = g_file_query_info(uri, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME, G_FILE_QUERY_INFO_NONE, NULL, &error);
+	utf8uri = g_file_info_get_display_name(ginfo);
+	g_object_unref(ginfo);
+	return utf8uri;
+}
+
+gchar *filename_utf8_from_uri(GFile *uri) {
+	gchar *utf8name, *slashpos;
+	
+	utf8name = full_path_utf8_from_uri(uri);
+	slashpos = strrchr(utf8name,'/');
+	if (slashpos) {
+		gchar *tmp;
+		slashpos++;
+		tmp = g_strdup(slashpos);
+		g_free(utf8name);
+		return tmp;
+	}
+	return utf8name;
+}*/
+
+/*gchar *get_utf8filename_from_on_disk_encoding(const gchar *encodedname) {
 	if (encodedname) {
 		GError *gerror=NULL;
 		gsize b_written;
@@ -79,7 +174,7 @@ gchar *get_utf8filename_from_on_disk_encoding(const gchar *encodedname) {
 		return ondiskencoding;
 	}
 	return NULL;
-}
+}*/
 
 gboolean string_is_color(const gchar *color) {
 	GdkColor gcolor;
@@ -139,27 +234,27 @@ gchar *filemode_to_string(mode_t statmode) {
  * trailing slash appended!!
  *
  * Return value: #gchar* newly allocated, or NULL
- */
+ * /
 gchar *return_root_with_protocol(const gchar *url) {
 	gchar *q;
 	if (!url) return NULL;
 	q = strchr(url,':');
 	if (q && *(q+1)=='/' && *(q+2)=='/' && *(q+3)!='\0') {
-		/* we have a protocol */
+		/ * we have a protocol * /
 		gchar *root = strchr(q+3, '/');
 		if (root) return g_strndup(url, root - url + 1);
-		/* if there is no third slash character, we probably
+		/ * if there is no third slash character, we probably
 		have an url like http://someserver so we will append 
-		the slash ourselves */
+		the slash ourselves * /
 		return g_strconcat(url, "/",NULL);
 	} else if (url[0] == '/') {
-		/* no protocol, return / */
+		/ * no protocol, return / * /
 		return g_strdup("/");
 	}
-	/* no root known */
+	/ * no root known * /
 	return NULL;
 }
-
+*/
 /**
  * pointer_switch_addresses:
  * a: #gpointer;
@@ -199,82 +294,6 @@ void list_switch_order(GList *first, GList *second) {
 	first->data = second->data;
 	second->data = tmp;
 }
-/**
- * file_copy:
- * @source: a #gchar * containing the source filename
- * @dest: a #gchar * containing the destination filename
- * 
- * copies the contents of the file source to dest
- * this function is Gnome-VFS aware, so it will work on URI's
- * 
- * Return value: gboolean, TRUE if the function succeeds
- **/
-#ifdef HAVE_GNOME_VFS
-#define BYTES_TO_PROCESS 8196
-gboolean file_copy(gchar *source, gchar *dest) {
-	GnomeVFSHandle *read_handle, *write_handle;
-	GnomeVFSFileSize bytes_read, bytes_written;
-	guint buffer[BYTES_TO_PROCESS];
-	GnomeVFSResult result;
-	gchar *OnDiEn_source, *OnDiEn_dest;
-	OnDiEn_source = get_filename_on_disk_encoding(source);
-	OnDiEn_dest = get_filename_on_disk_encoding(dest);
-	
-	result = gnome_vfs_open(&read_handle, OnDiEn_source, GNOME_VFS_OPEN_READ);
-	g_free(OnDiEn_source);
-	if (result != GNOME_VFS_OK) return FALSE;
-	result = gnome_vfs_create(&write_handle, OnDiEn_dest, GNOME_VFS_OPEN_WRITE, FALSE, 0644);
-	g_free(OnDiEn_dest);
-	if (result != GNOME_VFS_OK) {
-		gnome_vfs_close(read_handle);
-		return FALSE;
-	}
-	result = gnome_vfs_read (read_handle, buffer, BYTES_TO_PROCESS, &bytes_read);
-	while (result == GNOME_VFS_OK) {
-		result = gnome_vfs_write (write_handle, buffer, bytes_read, &bytes_written);
-		if (result != GNOME_VFS_OK || bytes_written != bytes_read) {
-			DEBUG_MSG("file_copy, return FALSE, write result=%d, written=%ld, read=%ld\n",result,(long)bytes_written,(long)bytes_read);
-			gnome_vfs_close(write_handle);
-			gnome_vfs_close(read_handle);
-			return FALSE;
-		}
-		result = gnome_vfs_read(read_handle, buffer, BYTES_TO_PROCESS, &bytes_read);
-	}
-	gnome_vfs_close(write_handle);
-	gnome_vfs_close(read_handle);
-	return TRUE;
-}
-#else  /* HAVE_GNOME_VFS */
-gboolean file_copy(gchar *source, gchar *dest) {
-#ifdef DEVELOPMENT
-	g_assert(source);
-	g_assert(dest);
-#endif
-	int c;
-	FILE *in, *out;
-	gchar *OnDiEn_source, *OnDiEn_dest;
-	OnDiEn_source = get_filename_on_disk_encoding(source);
-	OnDiEn_dest = get_filename_on_disk_encoding(dest);
-
-	in = fopen(OnDiEn_source, "r");
-	g_free(OnDiEn_source);
-	if (!in) {
-		return FALSE;
-	}
-	out = fopen(OnDiEn_dest, "w");
-	g_free(OnDiEn_dest);
-	if (!out) {
-		fclose(in);
-		return FALSE;
-	}
-	while((c=fgetc(in)) != EOF) {
-		fputc(c,out);
-	}
-	fclose(in);
-	fclose(out);
-	return TRUE;
-}
-#endif /* HAVE_GNOME_VFS */
 
 static gint length_common_prefix(gchar *first, gchar *second) {
 	gint i=0;
@@ -314,31 +333,6 @@ gint find_common_prefixlen_in_stringlist(GList *stringlist) {
 	}
 	return commonlen;
 }
-/**
- * append_string_to_file:
- * @filename: a #gchar * containing the destination filename
- * @string: a #gchar * containing the string to append
- * 
- * opens the file filename in append mode, and appends the string
- * no newline or anything else is appended, just the string
- *
- * DOES NOT YET SUPPORT GNOME_VFS !!!
- * 
- * Return value: gboolean, TRUE if the function succeeds
- **/
-gboolean append_string_to_file(gchar *filename, gchar *string) {
-	FILE *out;
-	gchar *ondiskencoding = get_filename_on_disk_encoding(filename);
-	out = fopen(ondiskencoding, "a");
-	g_free(ondiskencoding);
-	if (!out) {
-		DEBUG_MSG("append_to_file, could not open file %s for append\n", filename);
-		return FALSE;
-	}
-	fputs(string, out);
-	fclose(out);
-	return TRUE;
-}
 /**
  * countchars:
  * @string: a gchar * to count the chars in
@@ -348,7 +342,7 @@ gboolean append_string_to_file(gchar *filename, gchar *string) {
  * 
  * Return value: guint with the number of characters found
  **/
-guint countchars(const gchar *string, const gchar *chars) {
+static guint countchars(const gchar *string, const gchar *chars) {
 	guint count=0;
 	gchar *newstr = strpbrk(string, chars);
 	while(newstr) {
@@ -490,7 +484,7 @@ gchar *unexpand_string(const gchar *original, const char specialchar, Tconvert_t
 	Tconvert_table *tmpentry;
 	
 	orig = g_strdup(original);
-	DEBUG_MSG("original='%s', strlen()=%d\n",original,strlen(original));
+	DEBUG_MSG("original='%s', strlen()=%zd\n",original,strlen(original));
 	tosearchfor = g_malloc(tablesize(table)+1);
 	DEBUG_MSG("tablesize(table)=%d, alloc'ed %d bytes for tosearchfor\n",tablesize(table), tablesize(table)+1);
 	tmp = tosearchfor;
@@ -502,7 +496,7 @@ gchar *unexpand_string(const gchar *original, const char specialchar, Tconvert_t
 	}
 	*tmp = '\0';
 	DEBUG_MSG("unexpand_string, tosearchfor='%s'\n",tosearchfor);
-	DEBUG_MSG("alloc'ing %d bytes\n", (countchars(original, tosearchfor) + strlen(original) + 1));
+	DEBUG_MSG("alloc'ing %zd bytes\n", (countchars(original, tosearchfor) + strlen(original) + 1));
 	retval = g_malloc((countchars(original, tosearchfor) + strlen(original) + 1) * sizeof(gchar));
 	dest = retval;
 	prev = orig;
@@ -610,23 +604,25 @@ large php files, a cache of
 	12 resulted in 152% of the buffer to be parsed
 	14 resulted in 152% of the buffer to be parsed
 	16 resulted in 152% of the buffer to be parsed
-so we keep it at 10 for the moment
+so we keep it at 12 for the moment
 */
-#define UTF8_OFFSET_CACHE_SIZE 10
+#define UTF8_OFFSET_CACHE_SIZE 12
 /* #define UTF8_BYTECHARDEBUG */
 
 typedef struct {
-	gchar *last_buf;
 	/* the two arrays must be grouped and in this order, because they are moved back 
 	one position in ONE memmove() call */
-	guint  last_byteoffset[UTF8_OFFSET_CACHE_SIZE];
-	guint  last_charoffset[UTF8_OFFSET_CACHE_SIZE];
+	guint last_byteoffset[UTF8_OFFSET_CACHE_SIZE];
+	guint last_charoffset[UTF8_OFFSET_CACHE_SIZE];
 #ifdef UTF8_BYTECHARDEBUG
 	guint numcalls_since_reset;
 	unsigned long long int numbytes_parsed;
 	guint numcalls_cached_since_reset;
 	unsigned long long int numbytes_cached_parsed;
 #endif
+#ifdef DEVELOPMENT
+	gchar *last_buf;
+#endif
 } Tutf8_offset_cache;
 
 static Tutf8_offset_cache utf8_offset_cache;
@@ -635,13 +631,8 @@ static Tutf8_offset_cache utf8_offset_cache;
  * utf8_offset_cache_reset:
  * 
  * this function will reset the utf8 offset cache used by 
- * utf8_byteoffset_to_charsoffset_cached()
+ * utf8_byteoffset_to_charsoffset_cached() to use a new buffer
  *
- * normally this is done automatically if utf8_byteoffset_to_charsoffset_cached() 
- * is called with a new buffer. But if you ever call that function for 
- * the same buffer but the buffer is changed in the meantime you have 
- * to reset it manually using utf8_offset_cache_reset()
- * 
  * Return value: void
  **/
 #ifdef __GNUC__
@@ -654,6 +645,39 @@ void utf8_offset_cache_reset() {
 #endif
 	memset(&utf8_offset_cache, 0, sizeof(Tutf8_offset_cache));
 }
+
+guint utf8_charoffset_to_byteoffset_cached(const gchar *string, guint charoffset) {
+	guint i=UTF8_OFFSET_CACHE_SIZE-1, retval;
+	
+	if (charoffset ==0) return 0;
+	/*g_print("requested charoffset %d\n",charoffset);*/
+	while (i > 0 && utf8_offset_cache.last_charoffset[i] > charoffset) {
+		i--;
+	}
+	/*g_print("cache pos %d has charoffset %d and byteoffset %d\n",i,utf8_offset_cache.last_charoffset[i],utf8_offset_cache.last_byteoffset[i]);*/
+	if (i > 0) {
+		const gchar *tmp;
+		if (utf8_offset_cache.last_charoffset[i] == charoffset) {
+			return utf8_offset_cache.last_byteoffset[i];
+		}
+		tmp = string+utf8_offset_cache.last_byteoffset[i];
+		retval = (g_utf8_offset_to_pointer(tmp, charoffset-utf8_offset_cache.last_charoffset[i])-tmp)+utf8_offset_cache.last_byteoffset[i];
+		/*g_print("non-cached byteoffset %d, cached byteoffset %d for charoffset %d\n",(g_utf8_offset_to_pointer(string, charoffset)-string),retval,charoffset);*/
+	} else {
+		retval = (g_utf8_offset_to_pointer(string, charoffset)-string);
+		/*g_print("non-cached byteoffset %d for charoffset %d\n",retval,charoffset);*/
+	}
+	if (i == (UTF8_OFFSET_CACHE_SIZE-1)) {
+		/* add this new calculation to the cache */
+		/* this is a nasty trick to move all guint entries one back in the array, so we can add the new one */
+		memmove(&utf8_offset_cache.last_byteoffset[0], &utf8_offset_cache.last_byteoffset[1], (UTF8_OFFSET_CACHE_SIZE+UTF8_OFFSET_CACHE_SIZE-1)*sizeof(guint));
+		/*g_print("add byteoffset %d charoffset %d to position %d in the cache\n",retval,charoffset,UTF8_OFFSET_CACHE_SIZE-1);*/
+		utf8_offset_cache.last_byteoffset[UTF8_OFFSET_CACHE_SIZE-1] = retval;
+		utf8_offset_cache.last_charoffset[UTF8_OFFSET_CACHE_SIZE-1] = charoffset;
+	}
+	return retval;
+}
+
 /**
  * utf8_byteoffset_to_charsoffset_cached:
  * @string: the gchar * you want to count
@@ -670,32 +694,31 @@ void utf8_offset_cache_reset() {
  * 
  * Return value: guint with character offset
  **/
-guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset) {
+guint utf8_byteoffset_to_charsoffset_cached(const gchar *string, glong byteoffset) {
 	guint retval;
 	gint i = UTF8_OFFSET_CACHE_SIZE-1;
 	if (byteoffset ==0) return 0;
-
-	if (string != utf8_offset_cache.last_buf) {
-		utf8_offset_cache_reset();
-		utf8_offset_cache.last_buf = string;
+#ifdef DEVELOPMENT
+	if (utf8_offset_cache.last_buf != NULL && string != utf8_offset_cache.last_buf) {
+		/*utf8_offset_cache_reset();
+		utf8_offset_cache.last_buf = string;*/
+		g_print("bug found in a call to utf8_byteoffset_to_charsoffset_cached, the cache was not reset\n");
+		exit(156);
 	}
-#ifdef DEBUG
-	DEBUG_MSG("utf8_byteoffset_to_charsoffset_cached, string %p has strlen %d, looking for byteoffset %ld, starting in cache at i=%d\n", string, strlen(string),byteoffset,i);
 #endif
+	DEBUG_MSG("utf8_byteoffset_to_charsoffset_cached, string %p has strlen %zd, looking for byteoffset %ld, starting in cache at i=%d\n", string, strlen(string),byteoffset,i);
 
 	while (i > 0 && utf8_offset_cache.last_byteoffset[i] > byteoffset) {
 		i--;
 	}
-	
+
 	if (i > 0) {
 		if (utf8_offset_cache.last_byteoffset[i] == byteoffset) {
-#ifdef DEBUG
-	DEBUG_MSG("byteoffset %ld is in the cache at i=%d, returning %d\n",byteoffset,i,utf8_offset_cache.last_charoffset[i]);
-#endif
+			DEBUG_MSG("byteoffset %ld is in the cache at i=%d, returning %d\n",byteoffset,i,utf8_offset_cache.last_charoffset[i]);
 			return utf8_offset_cache.last_charoffset[i];
-       }
-       /* if the byteoffset is in the middle of a multibyte character, this line will fail (but
-	       we are not supposed to get called in the middle of a character)*/
+		}
+		/* if the byteoffset is in the middle of a multibyte character, this line will fail (but
+		we are not supposed to get called in the middle of a character)*/
 		retval = g_utf8_pointer_to_offset(string+utf8_offset_cache.last_byteoffset[i], string+byteoffset)+utf8_offset_cache.last_charoffset[i];
 #ifdef UTF8_BYTECHARDEBUG
 		utf8_offset_cache.numbytes_parsed += (byteoffset - utf8_offset_cache.last_byteoffset[i]);
@@ -990,9 +1013,10 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	eff_link_to_filepath = most_efficient_filename(g_strdup(link_to_filepath));
 	DEBUG_MSG("eff_current: '%s'\n",eff_current_filepath);
 	DEBUG_MSG("eff_link_to: '%s'\n",eff_link_to_filepath);
-	/* get the size of the directory of the current_filename */
-	current_filename_length = strlen(strrchr(eff_current_filepath, DIRCHR))-1;
-	link_to_filename_length = strlen(strrchr(eff_link_to_filepath, DIRCHR))-1;
+	/* get the size of the directory of the current_filename. we work on URI's
+	so we always use the / as character */
+	current_filename_length = strlen(strrchr(eff_current_filepath, '/'))-1;
+	link_to_filename_length = strlen(strrchr(eff_link_to_filepath, '/'))-1;
 	DEBUG_MSG("create_relative_link_to, filenames: current: %d, link_to:%d\n", current_filename_length,link_to_filename_length); 
 	current_dirname_length = strlen(eff_current_filepath) - current_filename_length;
 	link_to_dirname_length = strlen(eff_link_to_filepath) - link_to_filename_length;
@@ -1018,7 +1042,7 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	/* this is the common length, but we need the common directories */
 	if (eff_current_filepath[common_lenght] != DIRCHR) {
 		gchar *ltmp = &eff_current_filepath[common_lenght];
-		while ((*ltmp != DIRCHR) && (common_lenght > 0)) {
+		while ((*ltmp != '/') && (common_lenght > 0)) {
 			common_lenght--;
 			ltmp--;
 		}
@@ -1029,7 +1053,7 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	   is compared to the link_to_file, that is the amount of ../ we need to add */
 	deeper_dirs = 0;
 	for (count = common_lenght+1; count <= current_dirname_length; count++) {
-		if (eff_current_filepath[count] == DIRCHR) {
+		if (eff_current_filepath[count] == '/') {
 			deeper_dirs++;
 			DEBUG_MSG("create_relative_link_to, on count=%d, deeper_dirs=%d\n", count, deeper_dirs);
 		}
@@ -1049,66 +1073,19 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	g_free(eff_link_to_filepath);
 	return returnstring;
 }
-#ifdef HAVE_GNOME_VFS
-#define STRIP_FILE_URI
-#endif
-#ifdef HAVE_ATLEAST_GTK_2_4
-#define STRIP_FILE_URI
-#endif
-/**
- * create_full_path:
- * @filename: a gchar * with the (relative or not) filename
- * @basedir: a gchar * with a basedir or NULL for current dir
- *
- * if filename is already absolute, it returns it
- * else it will use basedir if available, else the current dir
- * to add to the filename to form the full path
- *
- * for URL's it will simply return a strdup(), except for file:// URL's, 
- * there the file:// bit is stripped and 
- * IF YOU HAVE GNOME_VFS any %XX sequenves are converted
- * so if you DON'T have gnome_vfs, you should not feed file:// uri's!!
- *
- * it does use most_efficient_filename() to remote unwanted dir/../ entries
- *
- * Return value: a newly allocated gchar * with the full path
- **/
-gchar *create_full_path(const gchar * filename, const gchar *basedir) {
-	gchar *absolute_filename;
-	gchar *tmpcdir;
 
-	if (!filename) return NULL;
-	DEBUG_MSG("create_full_path, filename=%s, basedir=%s\n", filename, basedir);
-#ifdef STRIP_FILE_URI
-	if (strchr(filename, ':') != NULL) { /* it is an URI!! */
-		DEBUG_MSG("create_full_path, %s is an URI\n",filename);
-		if (strncmp(filename, "file://", 7)==0) {
-#ifdef HAVE_GNOME_VFS
-			return gnome_vfs_get_local_path_from_uri(filename);
-#else
-			/* THIS IS A BUG, IF YOU DON'T HAVE GNOME_VFS BUT YOU DO HAVE 
-			GTK-2.4 A %21 OR SOMETHING LIKE THAT IS NOW NOT CONVERTED !!!!!!!!! */
-			return g_strdup(filename+7); /* file:// URI's are never relative paths */
-#endif
-		}
-		return g_strdup(filename); /* cannot do this on remote paths */
+/**
+ * strip_trailing_slash:
+ * @dirname: a gchar *pointing to a directory
+ *
+ * Return value: the same string, but 1 char shorter if there was a trailing slash
+ */
+gchar *strip_trailing_slash(gchar *input) {
+	if (input) {
+		gint len = strlen(input);
+		if (input[len-1] == '/') input[len-1] = '\0';
 	}
-#endif /* HAVE_GNOME_VFS */
-	if (g_path_is_absolute(filename)) {
-		absolute_filename = g_strdup(filename);
-	} else {
-		if (basedir) {
-			tmpcdir = ending_slash(basedir);
-		} else {
-			gchar *curdir = g_get_current_dir();
-			tmpcdir = ending_slash(curdir);
-			g_free(curdir);
-		}
-		absolute_filename = g_strconcat(tmpcdir, filename, NULL);
-		g_free(tmpcdir);
-	}
-	absolute_filename = most_efficient_filename(absolute_filename);
-	return absolute_filename;
+	return input;
 }
 
 /**
@@ -1152,52 +1129,21 @@ gchar *path_get_dirname_with_ending_slash(const gchar *filename) {
 }
 
 /**
- * file_exists_and_readable:
- * @filename: a #const gchar * with a file path
+ * full_path_exists:
+ * @full_path: gchar*
  *
- * tests if the file exists, and  if it is readable, the last
- * check is not reliable, it does not check all the groups you are
- * in, so change this function before you rely on that check!
+ * convenience function that will create a GFile, check if it exists, and unref it again.
  *
- * this function is Gnome-VFS aware, so it will work on URI's
- *
- * Return value: gboolean, TRUE if readable, else FALSE
- **/
-gboolean file_exists_and_readable(const gchar * filename) {
-	gchar *ondiskencoding;
-	gboolean retval=TRUE;
-#ifdef DEVELOPMENT
-	g_assert(filename);
-#endif
-	if (!filename || strlen(filename) < 2) {
-		DEBUG_MSG("file_exists_and_readable, strlen(filename) < 2 or no filename!!!!\n");
-		return FALSE;
-	}
-	DEBUG_MSG("file_exists_and_readable, filename(%p)=\"%s\", strlen(filename)=%d\n", filename, filename, strlen(filename));
-	ondiskencoding = get_filename_on_disk_encoding(filename);
-	DEBUG_MSG("file_exists_and_readable, ondiskencoding='%s'\n",ondiskencoding);
-#ifndef WIN32
-#ifdef HAVE_GNOME_VFS
-	{
-		GnomeVFSURI* uri;
-		uri = gnome_vfs_uri_new(ondiskencoding);
-		retval = gnome_vfs_uri_exists(uri);
-		DEBUG_MSG("gnome_vfs_uri has path %s\n",gnome_vfs_uri_get_path(uri));
-		gnome_vfs_uri_unref(uri);
-		DEBUG_MSG("file_exists_and_readable, return %d for %s\n",retval,filename);
-	}
-#else /* HAVE_GNOME_VFS */
-	{
-		struct stat naamstat;
-		errno = 0;
-		retval = ((stat(ondiskencoding, &naamstat) == 0) && (errno == 0));
-		DEBUG_MSG("file_exists_and_readable, retval=%d (ernno=%d) for %s\n",retval,errno,ondiskencoding);
-	}
-#endif /* HAVE_GNOME_VFS */
-	g_free(ondiskencoding);
-#endif /* WIN32 */
+ * /
+gboolean full_path_exists(const gchar *full_path) {
+	gboolean retval;
+	GFile *uri;
+	uri = g_file_new_for_path(full_path);
+	retval = g_file_query_exists(uri,NULL);
+	g_object_unref(uri);
 	return retval;
-}
+}*/
+
 /**
  * return_first_existing_filename:
  * @filename: a #const gchar * with a filename
@@ -1206,16 +1152,17 @@ gboolean file_exists_and_readable(const gchar * filename) {
  * you can pass multiple filenames to this function, and it will return
  * the first filename that really exists according to file_exists_and_readable()
  *
- * Return value: gchar * with the first filename found
+ * Return value: GFile * with the first filename found
  **/
-gchar *return_first_existing_filename(const gchar *filename, ...) {
+GFile *return_first_existing_filename(const gchar *filename, ...) {
 	va_list args;
-	gchar *retval=NULL;
+	GFile *retval=NULL;
 
 	va_start(args, filename);
 	while (filename) {
-		if (file_exists_and_readable(filename)) {
-			retval = g_strdup(filename);
+		DEBUG_MSG("return_first_existing_filename, testing %s\n",filename);
+		if (access(filename, R_OK) == 0) {
+			retval = g_file_new_for_path(filename);
 			break;
 		}
 		filename = va_arg(args, gchar*);
@@ -1234,12 +1181,16 @@ gchar *return_first_existing_filename(const gchar *filename, ...) {
  *
  * Return value: gboolean, TRUE if the file has one of the extensions in the array
  **/
-gboolean filename_test_extensions(gchar **extensions, gchar *filename) {
+gboolean filename_test_extensions(gchar **extensions, const gchar *filename) {
+	gint fnlen;
 	if (!extensions) {
 		return FALSE;
 	}
+	fnlen = strlen(filename);
 	while (*extensions) {
-		if (strncmp(&filename[strlen(filename)-strlen(*extensions)], *extensions, strlen(*extensions)) == 0 ) {
+		gint extlen;
+		extlen = strlen(*extensions);
+		if (fnlen > extlen && strncmp(filename + fnlen - extlen, *extensions, extlen) == 0 ) {
 			return TRUE;
 		}
 		extensions++;
@@ -1279,7 +1230,7 @@ gchar *bf_str_repeat(const gchar * str, gint number_of) {
  *
  * Return value: the found #gint, -1 on failure
  **/
-gint get_int_from_string(gchar *string) {
+gint get_int_from_string(const gchar *string) {
 	if (string) {
 		gint faktor = 1, result=-1;
 		gint i,len = strlen(string);
@@ -1303,6 +1254,61 @@ gint get_int_from_string(gchar *string) {
 	return -1;
 }
 
+gchar *unique_path(const gchar *basedir, const gchar *prefix, GHashTable *excludehash) {
+	gchar *path, *dir;
+	gint count=0;
+	if (!g_file_test(basedir, G_FILE_TEST_IS_DIR)) {
+		g_warning("dir %s does not exist\n",basedir);
+		return NULL;
+	}
+	dir = ending_slash(basedir);
+	if (prefix) {
+		path = g_strconcat(dir, prefix, NULL);
+	} else {
+		path = g_strdup_printf("%sbf-%d",dir,count);
+		count++;
+	}
+	while(g_file_test(path,G_FILE_TEST_EXISTS) || (excludehash && g_hash_table_lookup(excludehash, path)!=NULL )) {
+		g_free(path);
+		if (prefix)
+			path = g_strdup_printf("%s%s%d",dir,prefix,count);
+		else
+			path = g_strdup_printf("%sbf-%d",dir,count);
+		count++;
+	}
+	g_free(dir);
+	/*g_print("return %s\n",path);*/
+	return path;
+}
+
+static gchar *return_securedir(void) {
+	if (main_v->securedir) {
+		if (access(main_v->securedir, W_OK|X_OK)==0) {
+			return main_v->securedir;
+		} else {
+			g_free(main_v->securedir);
+		}
+	}
+	DEBUG_MSG("return_securedir,g_get_tmp_dir()=%s\n", g_get_tmp_dir());
+	/* it is SAFE to use tempnam() here, because we don't open a file with that name,
+	 * we create a directory with that name. mkdir() will FAIL if this name is a hardlink
+	 * or a symlink, so we DO NOT overwrite any file the link is pointing to
+	 */
+	main_v->securedir = unique_path(g_get_tmp_dir(), "bluefish", NULL);
+#ifndef WIN32
+	while (mkdir(main_v->securedir, 0700) != 0) {
+		g_free(main_v->securedir);
+		main_v->securedir = unique_path(g_get_tmp_dir(), "bluefish", NULL);
+	}
+#else
+	while (mkdir(main_v->securedir) != 0) {
+		g_free(main_v->securedir);
+		main_v->securedir = unique_path(g_get_tmp_dir(), "bluefish", NULL);
+	}
+#endif
+	return main_v->securedir;
+}
+
 /**
  * create_secure_dir_return_filename:
  *
@@ -1316,26 +1322,12 @@ gint get_int_from_string(gchar *string) {
  *
  * Return value: a newly allocated #gchar * containing a temporary filename in a secure dir
  **/
-gchar *create_secure_dir_return_filename() {
+gchar *create_secure_dir_return_filename(void) {
 	gchar *name, *name2;
-	DEBUG_MSG("create_secure_dir_return_filename,g_get_tmp_dir()=%s\n", g_get_tmp_dir());
-	/* it is SAFE to use tempnam() here, because we don't open a file with that name,
-	 * we create a directory with that name. mkdir() will FAIL if this name is a hardlink
-	 * or a symlink, so we DO NOT overwrite any file the link is pointing to
-	 */
-	name = tempnam(g_get_tmp_dir(), NULL);
-	DEBUG_MSG("create_secure_dir_return_filename, name=%s\n", name);
-	if (!name) {
-		return NULL;
-	}
-
-	if (mkdir(name, 0700) != 0) {
-		g_free(name);
-		return NULL;
-	}
-	name2 = tempnam(name, NULL);
+	name = return_securedir();
+	DEBUG_MSG("create_secure_dir_return_filename, dir is at %s\n",name);
+	name2 = unique_path(name, NULL, NULL);
 	DEBUG_MSG("create_secure_dir_return_filename, name2=%s\n", name2);
-	g_free(name);
 	return name2;
 }
 /**
@@ -1348,12 +1340,12 @@ gchar *create_secure_dir_return_filename() {
  *
  * Return value: void
  **/
-void remove_secure_dir_and_filename(gchar *filename) {
+/*void remove_secure_dir_and_filename(gchar *filename) {
 	gchar *dirname = g_path_get_dirname(filename);
 	unlink(filename);
 	rmdir(dirname);
 	g_free(dirname);
-}
+}*/
 
 /* gchar *buf_replace_char(gchar *buf, gint len, gchar srcchar, gchar destchar) {
 	gint curlen=0;
@@ -1430,15 +1422,15 @@ void wordcount(gchar *text, guint *chars, guint *lines, guint *words)
 	if(*chars > 0) (*lines)++;
 }
 
-GSList *gslist_from_glist(GList *src) {
+/*GSList *gslist_from_glist_reversed(GList *src) {
 	GSList *target=NULL;
 	GList *tmplist = g_list_first(src);
 	while (tmplist) {
-		target = g_slist_append(target, tmplist->data);
+		target = g_slist_prepend(target, tmplist->data);
 		tmplist = g_list_next(tmplist);
 	}
 	return target;
-}
+}*/
 
 GList *glist_from_gslist(GSList *src) {
 	GList *target=NULL;
@@ -1449,3 +1441,117 @@ GList *glist_from_gslist(GSList *src) {
 	}
 	return target;
 }
+/* returns a newly allocated string! */
+gchar *bf_portable_time(const time_t *timep) {
+	gchar *retstr=NULL;
+#ifdef HAVE_CTIME_R
+	retstr = g_new0(gchar, 128);
+	ctime_r(timep,retstr);
+#else
+#ifdef HAVE_ASCTIME_R
+	retstr = g_new0(gchar, 128);
+	asctime_r(localtime(timep),retstr);
+#else
+#ifdef HAVE_CTIME
+	retstr = g_strdup(ctime(timep));
+#else
+#ifdef HAVE_ASCTIME
+	retstr = g_strdup(asctime(localtime(timep)));
+#endif /* HAVE_ASCTIME */
+#endif /* HAVE_CTIME */
+#endif /* HAVE_ASCTIME_R */
+#endif /* HAVE_CTIME_R */
+	if (retstr) { /* one or more of the above functions end with a newline */
+		int len = strlen(retstr);
+		if (retstr[len-1]=='\n')
+			retstr[len-1]='\0';
+	}
+	return retstr;
+}
+
+/* these hash functions hash the first 3 strings in a gchar ** */
+gboolean arr3_equal (gconstpointer v1,gconstpointer v2)
+{
+	const gchar **arr1 = (const gchar **)v1;
+	const gchar **arr2 = (const gchar **)v2;
+  
+	return ((strcmp ((char *)arr1[0], (char *)arr2[0]) == 0 )&&
+			(strcmp ((char *)arr1[1], (char *)arr2[1]) == 0) &&
+			(strcmp ((char *)arr1[2], (char *)arr2[2]) == 0));
+}
+
+guint arr3_hash(gconstpointer v)
+{
+  /* modified after g_str_hash */
+  	const signed char **tmp = (const signed char **)v;
+	const signed char *p;
+	guint32 h = *tmp[0];
+	if (h) {
+		gshort i=0;
+		while (i<3) {
+			p = tmp[i];
+			for (p += 1; *p != '\0'; p++)
+				h = (h << 5) - h + *p;
+			i++;
+		}
+	}
+	return h;
+}
+
+gchar *gfile_display_name(GFile *uri, GFileInfo *finfo) {
+	gchar *retval;
+	if (finfo && g_file_info_has_attribute(finfo,G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME)) {
+		retval = g_strdup(g_file_info_get_display_name(finfo));
+		DEBUG_MSG("gfile_display_name, got %s from finfo\n",retval);
+	} else {
+		GFileInfo *finfo2;
+		
+		finfo2 = g_file_query_info(uri,G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,G_FILE_QUERY_INFO_NONE,NULL,NULL);
+		retval = g_strdup(g_file_info_get_display_name(finfo2));
+		DEBUG_MSG("gfile_display_name, queried uri %p, and got %s from finfo %p\n",uri,retval,finfo2);
+		g_object_unref(finfo2);
+	}
+	return retval;
+}
+
+gboolean gfile_uri_is_parent(GFile *parent, GFile *child, gboolean recursive) {
+	if (parent == NULL) {
+		g_print("gfile_uri_is_parent, parent=NULL ??\n");
+		return FALSE;
+	} else if (child == NULL) {
+		g_print("gfile_uri_is_parent, child=NULL ??\n");
+		return FALSE;
+	} else {
+		gboolean retval = FALSE;
+		GFile *tmp, *tmp2=child;
+		
+		g_object_ref(child);
+		do {
+			tmp = g_file_get_parent(tmp2);
+			g_object_unref(tmp2);
+			if (tmp == NULL) {
+				break;
+			}
+			retval = g_file_equal(tmp,parent);
+			tmp2 = tmp;
+		} while (recursive == TRUE && retval != TRUE);
+		if (tmp) {
+			g_object_unref(tmp);
+		}
+		return retval;
+	}
+}
+gchar *get_hostname_from_uri(GFile *uri) {
+	gchar *tmp, *begin, *end1, *end2, *end, *retval;
+	
+	tmp = g_file_get_uri(uri);
+	begin = strstr(tmp, "://");
+	begin += 3;
+	end1 = strchr(begin, ':');
+	end2 = strchr(begin, '/');
+	end = (end1 < end2) ? end1 : end2;
+	retval = g_strndup(begin, (end-begin));
+	g_free(tmp); 
+	return retval;
+}
+
