@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * outputbox.h - the output box
  *
- * Copyright (C) 2002 Olivier Sessink
+ * Copyright (C) 2005-2009 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,12 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-void init_output_box(Tbfwin *bfwin,GtkWidget *vbox);
-void outputbox(Tbfwin *bfwin,gchar *pattern, gint file_subpat, gint line_subpat, gint output_subpat, gchar *command, gboolean show_all_output);
+#ifndef __OUTPUTBOX_H_
+#define __OUTPUTBOX_H_
 
+/*void init_output_box(Tbfwin *bfwin,GtkWidget *vbox);*/
+void fill_output_box(gpointer data, gchar *string);
+void outputbox(Tbfwin *bfwin,gchar *pattern, gint file_subpat, gint line_subpat, gint output_subpat, gchar *command);
+void outputbox_cleanup(Tbfwin *bfwin);
+
+#endif /* __OUTPUTBOX_H */
