@@ -19,7 +19,7 @@
  */
 
 GtkWidget *new_pixmap(gint type);
-GList *default_icon_list(void);
+void set_default_icon(void);
 
 extern const guint8 pixmap_edit_tag[];
 extern const guint8 pixmap_abbr[];
@@ -79,8 +79,19 @@ extern const guint8 pixmap_frameset2[];
 extern const guint8 pixmap_framewhiz[];
 extern const guint8 pixmap_fref[];
 extern const guint8 pixmap_frdialog[];
+extern const guint8 pixmap_frdialog2[];
+extern const guint8 pixmap_frhide[];
 extern const guint8 pixmap_frinfo[];
 extern const guint8 pixmap_frsearch[];
+extern const guint8 pixmap_frlibrary[];
+extern const guint8 pixmap_frlibrary_ro[];
+extern const guint8 pixmap_frgroup[];
+extern const guint8 pixmap_frnote[];
+extern const guint8 pixmap_frfunction[];
+extern const guint8 pixmap_frtag[];
+extern const guint8 pixmap_frvar[];
+extern const guint8 pixmap_frcss[];
+extern const guint8 pixmap_frsnippet[];
 extern const guint8 pixmap_headings[];
 extern const guint8 pixmap_hrule[];
 extern const guint8 pixmap_image[];
