@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * project.c - project functionality
  *
- * Copyright (C) 2003-2006 Olivier Sessink
+ * Copyright (C) 2003-2010 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,54 +18,95 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-/* #define DEBUG */
+/*#define DEBUG*/
 
 #include <gtk/gtk.h>
-#include <stdlib.h>
 #include <string.h>
-#include <unistd.h>    /* unlink() */
+#include <stdlib.h>
 
 #include "bluefish.h"
+
+#ifdef MAC_INTEGRATION
+#include <ige-mac-integration.h>
+#endif
+
 #include "project.h"
-#include "bf_lib.h"
-#include "bookmark.h"
-#include "document.h"
-#include "filebrowser.h"
-#include "gtk_easy.h"
-#include "gui.h"
-#include "menu.h"
-#include "rcfile.h"
 #include "stringlist.h"
+#include "gui.h"
+#include "dialog_utils.h"
+#include "document.h"
+#include "gtk_easy.h"
+#include "rcfile.h"
+#include "bf_lib.h"
+#include "filebrowser2.h"
+#include "file_dialogs.h"
+#include "menu.h"
+#include "bookmark.h"
+#include "plugins.h"
+#include "preferences.h"
 
 static void free_session(Tsessionvars *session) {
+	/* call free_session for any of the plugins that have free_session defined */
+	bfplugins_session_cleanup(session);
 	free_stringlist(session->classlist);
 	free_stringlist(session->colorlist);
 	free_stringlist(session->targetlist);
 	free_stringlist(session->urllist);
 	free_stringlist(session->fontlist);
-	free_stringlist(session->dtd_cblist);
-	free_stringlist(session->headerlist);
 	free_stringlist(session->positionlist);
 	free_stringlist(session->searchlist);
 	free_stringlist(session->replacelist);
 	free_arraylist(session->bmarks);
+	free_stringlist(session->recent_files);
+	free_stringlist(session->recent_dirs);
+	g_free(session->opendir);
+	g_free(session->savedir);
+	g_free(session->encoding);
+	g_free(session->last_filefilter);
+	g_free(session->default_mime_type);
+	g_free(session->template);
+	g_free(session->webroot);
+	g_free(session->documentroot);
+	g_free(session->sync_local_uri);
+	g_free(session->sync_remote_uri);
+	g_free(session->convertcolumn_separator);
+	g_free(session->convertcolumn_fillempty);
+#ifdef HAVE_LIBENCHANT
+	g_free(session->spell_lang);
+#endif
 	g_free(session);
 }
 
-static void project_setup_initial_session(Tsessionvars *session) {
-	session->view_html_toolbar = main_v->session->view_html_toolbar;
-	session->view_custom_menu = main_v->session->view_custom_menu;
-	session->view_main_toolbar = main_v->session->view_main_toolbar;
-	session->view_left_panel = main_v->session->view_left_panel;
+static void project_setup_initial_session(Tsessionvars *session, gboolean before_parse) {
+	if (!before_parse) {
+		/* session parts that will be filled by rcfile_parse() will not be included
+		   if before parse is TRUE (this is thus only used for new projects) */
+		session->webroot = g_strdup(main_v->session->webroot);
+		session->documentroot = g_strdup(main_v->session->documentroot);
+		session->encoding = g_strdup(main_v->session->encoding);
+		session->opendir = g_strdup(main_v->session->opendir);
+		session->savedir = g_strdup(main_v->session->savedir);
+		session->last_filefilter = g_strdup(main_v->session->last_filefilter);
+		session->default_mime_type = g_strdup(main_v->session->default_mime_type);
+		session->template = g_strdup(main_v->session->template);
+#ifdef HAVE_LIBENCHANT
+		session->spell_lang = g_strdup(main_v->session->spell_lang);
+		session->spell_check_default = main_v->session->spell_check_default;
+#endif
+	}
+
+	/* THE NUBER OF BYTES THAT IS COPIED HERE MUST BE EQUAL TO THE NUMBER OF INTEGERS
+	IN THE STRUCT IN bluefish.h ELSE ALL WILL FAIL */
+	memcpy(session, main_v->session, 29 * sizeof(gint));
 }
 
-Tbfwin *project_is_open(gchar *filename) {
+Tbfwin *project_is_open(GFile *uri) {
 	GList *tmplist;
 	tmplist = g_list_first(main_v->bfwinlist);
 	while(tmplist){
-		if (BFWIN(tmplist->data)->project 
-				&& BFWIN(tmplist->data)->project->filename
-				&& strcmp(BFWIN(tmplist->data)->project->filename, filename)==0) {
+		if (BFWIN(tmplist->data)->project
+				&& BFWIN(tmplist->data)->project->uri
+				&& g_file_equal( BFWIN(tmplist->data)->project->uri, uri )) {
 			return BFWIN(tmplist->data);
 		}
 		tmplist = g_list_next(tmplist);
@@ -74,20 +115,67 @@ Tbfwin *project_is_open(gchar *filename) {
 }
 
 static void update_project_filelist(Tbfwin *bfwin, Tproject *prj) {
+	GList *tmplist;
 	DEBUG_MSG("update_project_filelist, started, bfwin=%p, prj=%p\n",bfwin,prj);
 	free_stringlist(prj->files);
 	DEBUG_MSG("update_project_filelist, old list free'd, creating new list from documentlist %p (len=%d)\n",bfwin->documentlist,g_list_length(bfwin->documentlist));
-	prj->files = return_filenamestringlist_from_doclist(bfwin->documentlist);
+	tmplist = return_urilist_from_doclist(bfwin->documentlist);
+	prj->files = urilist_to_stringlist(tmplist);
+	free_urilist(tmplist);
+}
+
+
+void set_project_menu_widgets(Tbfwin *bfwin, gboolean win_has_project) {
+	menuitem_set_sensitive(bfwin->menubar, "/Project/Save", win_has_project);
+	menuitem_set_sensitive(bfwin->menubar, "/Project/Save as...", win_has_project);
+	menuitem_set_sensitive(bfwin->menubar, "/Project/Save & close", win_has_project);
+	menuitem_set_sensitive(bfwin->menubar, "/Project/Edit Project Options...", win_has_project);
 }
 
 static void setup_bfwin_for_project(Tbfwin *bfwin) {
-	bfwin->session = bfwin->project->session;
-	bfwin->bookmarkstore = bfwin->project->bookmarkstore;
+	DEBUG_MSG("setup_bfwin_for_project, bfwin=%p, bfwin->project=%p, bfwin->session=%p\n",bfwin,bfwin->project,bfwin->session);
+	DEBUG_MSG("setup_bfwin_for_project, bfwin->project->session=%p\n",bfwin->project->session);
+	bfwin->bmarkdata = bfwin->project->bmarkdata;
 	bmark_set_store(bfwin);
 	bmark_reload(bfwin);
-	filebrowser_set_basedir(bfwin, bfwin->project->basedir);
-	recent_menu_from_list(bfwin, bfwin->project->session->recent_files, FALSE);
+
+	gui_apply_session(bfwin);
 	set_project_menu_widgets(bfwin, TRUE);
+#ifdef MAC_INTEGRATION
+	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));
+#endif
+
+}
+
+static void setup_bfwin_for_nonproject(Tbfwin *bfwin) {
+	DEBUG_MSG("setup_bfwin_for_nonproject, bfwin=%p\n",bfwin);
+	bfwin->session = main_v->session;
+	bfwin->bmarkdata = main_v->bmarkdata;
+	bfwin->project = NULL;
+	bmark_set_store(bfwin);
+	/* normally there is always a current_document, but this function might be called in the transition
+	after all documents are just closed */
+	if (bfwin->current_document) gui_set_title(bfwin, bfwin->current_document, 0);
+
+	gui_apply_session(bfwin);
+	set_project_menu_widgets(bfwin, FALSE);
+#ifdef MAC_INTEGRATION
+	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));
+#endif
+}
+
+static void project_destroy(Tproject *project) {
+	DEBUG_MSG("project_destroy, project=%p, project->session=%p\n",project,project->session);
+	bookmark_data_cleanup(project->bmarkdata);
+	if (project->files) 
+		free_stringlist(project->files);
+	if (project->session)
+		free_session(project->session);
+	if (project->uri)
+		g_object_unref(project->uri);
+	if (project->name)
+		g_free(project->name);
+	g_free(project);
 }
 
 /* bfwin is allowed to be NULL for an empty project */
@@ -95,8 +183,8 @@ static Tproject *create_new_project(Tbfwin *bfwin) {
 	Tproject *prj;
 	prj = g_new0(Tproject,1);
 	prj->name = g_strdup(_("New project"));
-	prj->bookmarkstore = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_POINTER); 
-	DEBUG_MSG("create_new_project, project=%p, bookmarkstore=%p, bfwin=%p\n",prj,prj->bookmarkstore,bfwin);
+	prj->bmarkdata = bookmark_data_new();
+	DEBUG_MSG("create_new_project, project=%p, bfwin=%p\n",prj,bfwin);
 	if (bfwin) {
 		DEBUG_MSG("create_new_project, new project for bfwin %p\n",bfwin);
 		update_project_filelist(bfwin,prj);
@@ -105,9 +193,8 @@ static Tproject *create_new_project(Tbfwin *bfwin) {
 		DEBUG_MSG("create_new_project, new project, no bfwin\n");
 	}
 	prj->session = g_new0(Tsessionvars,1);
-	project_setup_initial_session(prj->session);
+	project_setup_initial_session(prj->session, FALSE);
 
-	/* we should copy bookmarks from the files to this session */
 	if (bfwin && prj->files) {
 		GList *tmplist;
 		tmplist = g_list_first(bfwin->documentlist);
@@ -115,7 +202,7 @@ static Tproject *create_new_project(Tbfwin *bfwin) {
 			bmark_clean_for_doc(DOCUMENT(tmplist->data));
 			tmplist = g_list_next(tmplist);
 		}
-	
+
 		tmplist = g_list_first(bfwin->session->bmarks);
 		while (tmplist) {
 			gchar **entry = (gchar**)tmplist->data;
@@ -135,25 +222,8 @@ static Tproject *create_new_project(Tbfwin *bfwin) {
 			tmplist = g_list_next(tmplist);
 		}
 	}
-	if (prj->files) {
-		gint len;
-		gchar *somefile, *prefix;
-		len = find_common_prefixlen_in_stringlist(prj->files);
-		somefile = (gchar *)prj->files->data;
-		prefix = g_strndup(somefile, len);
-		if (prefix[strlen(prefix)-1] == '/') {
-			prj->basedir = g_strdup(prefix);
-		} else {
-			prj->basedir = g_path_get_dirname(prefix);
-		}
-		g_free(prefix);
-	} else {
-		prj->basedir = g_strdup("");
-	}
-	prj->webdir = g_strdup("");
-	prj->template = g_strdup("");
-	prj->word_wrap = main_v->props.word_wrap;
 	if (bfwin) {
+		bfwin->session = prj->session;
 		setup_bfwin_for_project(bfwin);
 	}
 	return prj;
@@ -171,83 +241,76 @@ gboolean project_save(Tbfwin *bfwin, gboolean save_as) {
 	DEBUG_MSG("project_save, project=%p, num files was %d\n", bfwin->project, g_list_length(bfwin->project->files));
 	update_project_filelist(bfwin, bfwin->project);
 /*	bfwin->project->recentfiles = limit_stringlist(bfwin->project->recentfiles, main_v->props.max_recent_files, TRUE);*/
-	
+
 	bfwin->project->session->searchlist = limit_stringlist(bfwin->project->session->searchlist, 10, TRUE);
 	bfwin->project->session->replacelist = limit_stringlist(bfwin->project->session->replacelist, 10, TRUE);
 	bfwin->project->session->urllist = limit_stringlist(bfwin->project->session->urllist, 10, TRUE);
 	bmark_store_all(bfwin);
 
-	if (save_as || bfwin->project->filename == NULL) {
+	if (save_as || bfwin->project->uri == NULL) {
 		gint suflen,filen;
-		gchar *filename = NULL;
-#ifdef HAVE_ATLEAST_GTK_2_4
-		{
-			GtkWidget *dialog;
-/*			dialog = gtk_file_chooser_dialog_new(_("Enter Bluefish project filename"),NULL,
-					GTK_FILE_CHOOSER_ACTION_OPEN,
-					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-					NULL);*/
-			dialog = file_chooser_dialog(bfwin,_("Enter Bluefish project filename"),GTK_FILE_CHOOSER_ACTION_SAVE, NULL, TRUE, FALSE, "bfproject");
-			if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
-				filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+		GtkWidget *dialog;
+		GFile *newuri=NULL;
+		dialog = file_chooser_dialog(bfwin,_("Enter Bluefish project filename"),GTK_FILE_CHOOSER_ACTION_SAVE, NULL, TRUE, FALSE, "bfproject", FALSE);
+		if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+			gchar *filename;
+			newuri = gtk_file_chooser_get_file(GTK_FILE_CHOOSER(dialog));
+			filename = g_file_get_parse_name(newuri);
+			suflen = strlen(main_v->props.project_suffix);
+			filen = strlen(filename);
+			if (filen < suflen || strcmp(&filename[filen - suflen], main_v->props.project_suffix)!=0) {
+				GFile *tmp2;
+				gchar *tmp = g_strconcat(filename, main_v->props.project_suffix,NULL);
+				tmp2 = g_file_parse_name(tmp);
+				g_free(tmp);
+				g_object_unref(newuri);
+				newuri = tmp2;
 			}
-			gtk_widget_destroy(dialog);
+			g_free(filename);
 		}
-#else
-		filename = return_file_w_title(NULL, _("Enter Bluefish project filename"));
-#endif
-		if (!filename) {
+		gtk_widget_destroy(dialog);
+		if (!newuri) {
 			return FALSE;
 		}
-		if (save_as || bfwin->project->filename == NULL) {
-			gchar *ondiskencoding = get_filename_on_disk_encoding(filename);
-			if (g_file_test(ondiskencoding, G_FILE_TEST_EXISTS)) {
-				gchar *tmpstr;
+		if (save_as || bfwin->project->uri == NULL) {
+			gboolean exists;
+			exists = g_file_query_exists(newuri,NULL);
+			if (exists) {
+				gchar *tmpstr,*filename;
 				gint retval;
-				gchar *options[] = {_("_Cancel"), _("_Overwrite"), NULL};
+				const gchar *buttons[] = {_("_Cancel"), _("_Overwrite"), NULL};
+				filename = g_file_get_parse_name(newuri);
 				tmpstr = g_strdup_printf(_("A file named \"%s\" already exists."), filename);
-				retval = multi_warning_dialog(bfwin->main_window,tmpstr, 
-												_("Do you want to replace the existing file?"), 1, 0, options);
+				retval = message_dialog_new_multi(bfwin->main_window,
+															 GTK_MESSAGE_WARNING,
+															 buttons,
+															 tmpstr,
+															 _("Do you want to replace the existing file?"));
 				g_free(tmpstr);
+				g_free(filename);
 				if (retval == 0) {
-					g_free(filename);
-					g_free(ondiskencoding);
+					g_object_unref(newuri);
 					return FALSE;
 				}
 			}
-			g_free(ondiskencoding);
-		}
-		suflen = strlen(main_v->props.project_suffix);
-		filen = strlen(filename);
-		if (filen > suflen && strcmp(&filename[filen - suflen], main_v->props.project_suffix)==0) {
-			bfwin->project->filename = filename;
-		} else {
-			bfwin->project->filename = g_strconcat(filename, main_v->props.project_suffix,NULL);
-			g_free(filename);
 		}
+		bfwin->project->uri = newuri;
 	}
-	
-	DEBUG_MSG("project_save, saving project %p to file %s\n",bfwin->project,bfwin->project->filename);
-	retval = rcfile_save_project(bfwin->project, bfwin->project->filename);
-	add_to_recent_list(bfwin,bfwin->project->filename, FALSE, TRUE);
+
+	retval = rcfile_save_project(bfwin->project, bfwin->project->uri);
+	DEBUG_MSG("project_save, retval=%d\n",retval);
+	add_to_recent_list(bfwin,bfwin->project->uri, FALSE, TRUE);
 	return retval;
 }
 
-void set_project_menu_widgets(Tbfwin *bfwin, gboolean win_has_project) {
-	menuitem_set_sensitive(bfwin->menubar, "/Project/Save", win_has_project);
-	menuitem_set_sensitive(bfwin->menubar, "/Project/Save as...", win_has_project);
-	menuitem_set_sensitive(bfwin->menubar, "/Project/Save & close", win_has_project);
-	menuitem_set_sensitive(bfwin->menubar, "/Project/Edit Project Options...", win_has_project);
-}
-
-void project_open_from_file(Tbfwin *bfwin, gchar *fromfilename) {
+void project_open_from_file(Tbfwin *bfwin, GFile *fromuri) {
 	Tbfwin *prwin;
 	Tproject *prj;
 	gboolean retval;
+	GList *tmplist;
 
 	/* first we test if the project is already open */
-	prwin = project_is_open(fromfilename);
+	prwin = project_is_open(fromuri);
 	if (prwin != NULL) {
 		DEBUG_MSG("project_open_from_file, project is open in bfwin=%p\n",prwin);
 		gtk_window_present(GTK_WINDOW(prwin->main_window));
@@ -256,180 +319,124 @@ void project_open_from_file(Tbfwin *bfwin, gchar *fromfilename) {
 
 	prj = g_new0(Tproject,1);
 	prj->session = g_new0(Tsessionvars,1);
-	project_setup_initial_session(prj->session);
-	prj->bookmarkstore = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_POINTER); 
-	retval = rcfile_parse_project(prj, fromfilename);
+	DEBUG_MSG("project_open_from_file, project=%p, session=%p\n",prj,prj->session);
+	project_setup_initial_session(prj->session, TRUE);
+	prj->bmarkdata = bookmark_data_new();
+	retval = rcfile_parse_project(prj, fromuri);
 	if (!retval) {
-		DEBUG_MSG("project_open_from_file, failed parsing the project at file %s\n",fromfilename);
-		g_object_unref(prj->bookmarkstore);
+		DEBUG_MSG("project_open_from_file, failed parsing the project\n");
+		project_destroy(prj);
+		/* TODO: use project_destroy() 
+		bookmark_data_cleanup(prj->bmarkdata);
 		g_free(prj->session);
-		g_free(prj);
+		g_free(prj);*/
 		return;
 	}
-	add_to_recent_list(bfwin,fromfilename, FALSE, TRUE);
-	prj->filename = g_strdup(fromfilename);
-	DEBUG_MSG("project_open_from_file, basedir=%s\n",prj->basedir);
+	add_to_recent_list(bfwin,fromuri, FALSE, TRUE);
+	prj->uri = fromuri;
+	g_object_ref(fromuri);
 	if (bfwin->project == NULL && test_only_empty_doc_left(bfwin->documentlist)) {
 		/* we will use this Bluefish window to open the project */
 		prwin = bfwin;
-		/* now we need to clean the session, and reset it to the session from the project */
-		/* free_session(bfwin->session); there is no session specific to a window anymore, only a global one*/
-		bfwin->session = prj->session;
 		prwin->project = prj;
-		prwin->bookmarkstore = prj->bookmarkstore;
-		
-      gui_set_html_toolbar_visible(prwin, prj->session->view_html_toolbar, TRUE);
-      gui_set_main_toolbar_visible(prwin, prj->session->view_main_toolbar, TRUE);
-      gui_set_custom_menu_visible(prwin, prj->session->view_custom_menu, TRUE);
-		DEBUG_MSG("project_open_from_file, calling left_panel_show_hide_toggle bfwin=%p\n",prwin);
-		left_panel_show_hide_toggle(prwin,FALSE,prj->session->view_left_panel, TRUE);
-		filebrowser_set_basedir(prwin, prj->basedir);
-		DEBUG_MSG("project_open_from_file, calling docs_new_from_files for existing bfwin=%p\n",prwin);
-		docs_new_from_files(prwin, prj->files, TRUE);
+		prwin->session = prj->session;
+		DEBUG_MSG("project_open_from_file, project %p will be in existing prwin=%p\n",prj,bfwin);
+		/* destroy the current empty document, it should use settings from the new session */
+		doc_destroy(bfwin->current_document, TRUE);
 	} else {
 		/* we will open a new Bluefish window for this project */
 		DEBUG_MSG("project_open_from_file, we need a new window\n");
-		prwin = gui_new_window(prj->files, prj);
-		DEBUG_MSG("project_open_from_file, new window with files ready\n");
-		gui_set_title(prwin, prwin->current_document);
+		prwin = gui_new_window(prj);
 	}
-
-	set_project_menu_widgets(prwin, TRUE);
-	recent_menu_init_project(prwin);
-   bmark_reload(bfwin);
-   bmark_set_store(bfwin);
-}
-
-static void project_open(Tbfwin *bfwin) {
-	/* first we ask for a filename */
-	gchar *filename = NULL;
-#ifdef HAVE_ATLEAST_GTK_2_4
-	{
-		GtkWidget *dialog;
-		/*dialog = gtk_file_chooser_dialog_new(_("Select Bluefish project filename"),NULL,
-				GTK_FILE_CHOOSER_ACTION_OPEN,
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-				NULL);*/
-		dialog = file_chooser_dialog(bfwin, _("Select Bluefish project filename"), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, TRUE, FALSE, "bfproject");
-		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
-			filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+	tmplist = g_list_last(prj->files);
+	if (tmplist) {
+		while (tmplist) {
+			GFile *uri;
+			if (strstr ((gchar *) tmplist->data, "://") == NULL)
+				uri = g_file_new_for_path((gchar *) tmplist->data);
+			else
+				uri = g_file_new_for_uri((gchar *) tmplist->data);
+			doc_new_from_uri(prwin, uri, NULL, !(prj->files->prev==NULL), TRUE, -1, -1);
+			g_object_unref(uri);
+			tmplist = g_list_previous(tmplist);
 		}
-		gtk_widget_destroy(dialog);
-	}
-#else
-	filename = return_file_w_title(NULL, _("Select Bluefish project filename"));
-#endif
-	if (filename) {
-		DEBUG_MSG("project_open, for filename %s\n",filename);
-		project_open_from_file(bfwin,filename);
-		g_free(filename);
 	} else {
-		DEBUG_MSG("project_open, no filename.., returning\n");
+		doc_new(bfwin, FALSE);
+	}
+	DEBUG_MSG("project_open_from_file, new window with files ready at prwin=%p\n",prwin);
+	setup_bfwin_for_project(prwin);
+	recent_menu_init_project(prwin);
+	DEBUG_MSG("project_open_from_file, done\n");
+}
+
+static void project_open_response_lcb(GtkDialog *dialog,gint response,Tbfwin *bfwin) {
+	if (response == GTK_RESPONSE_ACCEPT) {
+		GFile *file;
+		file = gtk_file_chooser_get_file(GTK_FILE_CHOOSER(dialog));
+		project_open_from_file(bfwin,file);
+		g_object_unref(file);
+	}
+	gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+static void project_open(Tbfwin *bfwin) {
+	GtkWidget *dialog;
+	dialog = file_chooser_dialog(bfwin, _("Select Bluefish project filename"), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, TRUE, FALSE, "bfproject", FALSE);
+	g_signal_connect(dialog, "response", G_CALLBACK(project_open_response_lcb), bfwin);
+	gtk_widget_show_all(dialog);
+}
+
+void project_save_and_mark_closed(Tbfwin *bfwin) {
+	if (bfwin->project) {
+		project_save(bfwin, FALSE);
+		add_to_recent_list(bfwin,bfwin->project->uri, TRUE, TRUE);
+		bfwin->project->close=TRUE;
 	}
 }
 
-static void project_destroy(Tproject *project) {
-	g_object_unref(G_OBJECT(project->bookmarkstore));
-	free_stringlist(project->files);
-	free_session(project->session);
-	g_free(project->filename);
-	g_free(project->name);
-	g_free(project->basedir);
-	g_free(project->webdir);
-	g_free(project->template);
-	g_free(project);
-}
-
-static void setup_bfwin_for_nonproject(Tbfwin *bfwin) {
-	bfwin->session = main_v->session;
-	bfwin->bookmarkstore = main_v->bookmarkstore;
-	bfwin->project = NULL;
-	bmark_set_store(bfwin);
-	gui_set_title(bfwin, bfwin->current_document);
-	filebrowser_set_basedir(bfwin, NULL);
-	recent_menu_from_list(bfwin, main_v->session->recent_files, FALSE);
-	set_project_menu_widgets(bfwin, FALSE);
-}
-
-/* 
- * returns TRUE if the project is closed, 
- * returns FALSE if something went wrong or was cancelled
- */
-gboolean project_save_and_close(Tbfwin *bfwin) {
-	gboolean dont_save = FALSE;
-	while (!bfwin->project->filename) {
-		gchar *text;
-		gint retval;
-		gchar *buttons[] = {_("Do_n't save"), GTK_STOCK_CANCEL, GTK_STOCK_SAVE, NULL};
-		if (dont_save) {
-			break;
-		}
-		DEBUG_MSG("project_save_and_close, project not named, getting action\n");
-		/* dialog */
-
-		text = g_strdup(_("Do you want to save the project?"));
-		retval = multi_query_dialog(bfwin->main_window, text, 
-			_("If you don't save your changes they will be lost."), 2, 1, buttons);
-		switch (retval) {
-		case 0:
-			/* don't save proj. save files, though */
-			DEBUG_MSG("project_save_and_close, don't save project, but save files\n");
-			dont_save = TRUE;
-		break;
-		case 1:
-			/* cancel */
-			DEBUG_MSG("project_save_and_close, not closing window any more");
-			return FALSE;
-		break;
-		case 2:
-			DEBUG_MSG("project_save_and_close, bringing up save project dialog\n");
-			dont_save = project_save(bfwin, FALSE);
-		break;
-		default:
-		break;
-		}		
-	}
-	/* test if we should save */
-	if (!dont_save) {
-		gchar *backupfile = NULL;
-		if (!project_save(bfwin, FALSE)) {
-			DEBUG_MSG("project_save failed, returning\n");
-			return FALSE;
-		}
-		add_to_recent_list(bfwin,bfwin->project->filename, TRUE, TRUE);
-		
-        if (main_v->props.backup_cleanuponclose) {
-            backupfile = g_strconcat (bfwin->project->filename, main_v->props.backup_filestring, NULL);
-            if (file_exists_and_readable(backupfile)) {
-                gchar * ondiskencodingbckup = get_filename_on_disk_encoding(backupfile);
-#ifdef HAVE_GNOME_VFS
-                gnome_vfs_unlink(ondiskencodingbckup);
-#else
-		        unlink(ondiskencodingbckup);				
-#endif
-                g_free (ondiskencodingbckup);
-            }
-            g_free (backupfile);
-        }
-	}
-	bfwin_close_all_documents(bfwin, TRUE);
-	if (!test_only_empty_doc_left(bfwin->documentlist)) {
-		DEBUG_MSG("closing all documents failed, returning\n");
-		return FALSE;
+gboolean project_final_close(Tbfwin *bfwin, gboolean close_win) {
+	if (!bfwin->project)
+		return TRUE;
+	if (!close_win) {
+		add_to_recent_list(bfwin,bfwin->project->uri, TRUE, TRUE);
 	}
 	project_destroy(bfwin->project);
-	setup_bfwin_for_nonproject(bfwin);
-	DEBUG_MSG("project_save_and_close, returning TRUE\n");
+	/* we should only set the window for nonproject if the window will keep alive*/
+	if (!close_win) {
+		setup_bfwin_for_nonproject(bfwin);
+	}
 	return TRUE;
 }
 
+/*
+ * returns TRUE if the project is closed,
+ * returns FALSE if something went wrong or was cancelled
+ */
+/*gboolean project_save_and_close(Tbfwin *bfwin, gboolean close_win) {
+	if (!bfwin->project)
+		return TRUE;
+	DEBUG_MSG("project_save_and_close, bfwin=%p, project=%p, close_win=%d, project->close=%d\n",bfwin,bfwin->project,close_win,bfwin->project->close);
+
+	project_save(bfwin, FALSE);
+
+	if (test_only_empty_doc_left(bfwin->documentlist)) {
+		project_destroy(bfwin->project);
+		setup_bfwin_for_nonproject(bfwin);
+	} else {
+		gboolean retval;
+		/ * the last document that closes should close the window, and in the window-delete-event handler (in gui.c)
+		project_save_and_close() is called again, which will clean-up the memory * /
+		DEBUG_MSG("project_save_and_close, documents are closing!, setting project->close to TRUE for project %p in bfwin %p\n", bfwin->project, bfwin);
+		retval = doc_close_multiple_backend(bfwin, close_win);
+		if (retval) {
+			bfwin->project->close = TRUE;
+		}
+		return retval;
+	}
+	return TRUE;
+}*/
+
 typedef enum {
 	name,
-	basedir,
-	webdir,
-	template,
-	word_wrap,
 	projecteditor_entries_num
 } Tprojecteditor_entries;
 
@@ -437,6 +444,7 @@ typedef struct {
 	GtkWidget *win;
 	Tbfwin *bfwin;
 	Tproject *project;
+	Tsessionprefs sprefs;
 	GtkWidget *entries[projecteditor_entries_num];
 	gboolean destroy_project_on_close;
 } Tprojecteditor;
@@ -445,10 +453,11 @@ static void project_edit_destroy_lcb(GtkWidget *widget, Tprojecteditor *pred) {
 	DEBUG_MSG("project_edit_destroy_lcb, called for pred=%p\n",pred);
 /*	gtk_widget_destroy(pred->win);*/
 	if (pred->destroy_project_on_close) {
-		project_destroy(pred->project);
 		if (pred->bfwin) {
 			setup_bfwin_for_nonproject(pred->bfwin);
 		}
+		project_destroy(pred->project);
+		pred->project=NULL;
 	}
 	if (pred->project) {
 		pred->project->editor = NULL;
@@ -457,34 +466,26 @@ static void project_edit_destroy_lcb(GtkWidget *widget, Tprojecteditor *pred) {
 }
 
 static void project_edit_cancel_clicked_lcb(GtkWidget *widget, Tprojecteditor *pred) {
+
 	gtk_widget_destroy(pred->win);
 }
 
 static void project_edit_ok_clicked_lcb(GtkWidget *widget, Tprojecteditor *pred) {
 	Tproject *prj = pred->project;
-	
+
 	pred->destroy_project_on_close = FALSE;
-	
+
 	gtk_widget_hide(pred->win);
 	DEBUG_MSG("project_edit_ok_clicked_lcb, Tproject at %p, bfwin at %p\n",prj,pred->bfwin);
 	string_apply(&prj->name, pred->entries[name]);
-	string_apply(&prj->basedir, pred->entries[basedir]);
-	if (prj->basedir && strlen(prj->basedir)) {
-		gchar *tmp = prj->basedir;
-		prj->basedir = ending_slash(prj->basedir);
-		g_free(tmp);
-	}
-	string_apply(&prj->webdir, pred->entries[webdir]);
-	string_apply(&prj->template, pred->entries[template]);
-	integer_apply(&prj->word_wrap, pred->entries[word_wrap], TRUE);
-	DEBUG_MSG("project_edit_ok_clicked_lcb, name=%s, basedir=%s, webdir=%s\n",prj->name,prj->basedir,prj->webdir);
-
+	sessionprefs_apply(&pred->sprefs, pred->project->session);
 	if (pred->bfwin == NULL) {
-		pred->bfwin = gui_new_window(NULL, pred->project);
+		pred->bfwin = gui_new_window(pred->project);
+		pred->bfwin->session = pred->project->session;
+		setup_bfwin_for_project(pred->bfwin);
 	} else {
-		gui_set_title(pred->bfwin, pred->bfwin->current_document);
+		gui_set_title(pred->bfwin, pred->bfwin->current_document, 0);
 	}
-	setup_bfwin_for_project(pred->bfwin);
 /* set_project_menu_widgets(pred->bfwin, TRUE);*/
 	project_save(pred->bfwin,FALSE);
 	gtk_widget_destroy(pred->win);
@@ -523,55 +524,47 @@ void project_edit(Tbfwin *bfwin) {
 	DEBUG_MSG("project_edit, Tproject at %p\n",pred->project);
 	pred->bfwin = bfwin;
 	pred->project->editor = pred;
-	
+
 	pred->win = window_full2(wintitle, GTK_WIN_POS_CENTER_ALWAYS, 5
 			, G_CALLBACK(project_edit_destroy_lcb), pred, TRUE, NULL);
 	vbox = gtk_vbox_new(FALSE,0);
 	gtk_container_add(GTK_CONTAINER(pred->win),vbox);
-	
+
 	if (wintitle) {
 		g_free(wintitle);
 	}
-	
+
 	label = gtk_label_new(NULL);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
 	gtk_label_set_line_wrap(GTK_LABEL(label),TRUE);
 	{
 		gchar *message;
-		message = g_strdup_printf(_("This project contains %d files"), g_list_length(pred->project->files));
+		message = g_strdup_printf(
+			ngettext("This project contains %d file", "This project contains %d files", g_list_length(pred->project->files)),
+			g_list_length(pred->project->files));
 		gtk_label_set_markup(GTK_LABEL(label), message);
 		g_free(message);
 	}
-	
+
 	table = gtk_table_new (5, 4, FALSE);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 6);	
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 12);
-	
+
 	pred->entries[name] = entry_with_text(pred->project->name, 255);
 	gtk_widget_set_size_request(GTK_WIDGET(pred->entries[name]), 250, -1);
 	bf_mnemonic_label_tad_with_alignment(_("Project _Name:"), pred->entries[name], 1, 0.5, table, 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(table), pred->entries[name], 2, 3, 0, 1);
 
-	pred->entries[basedir] = entry_with_text(pred->project->basedir, 255);
-	bf_mnemonic_label_tad_with_alignment(_("Base _Dir:"), pred->entries[basedir], 1, 0.5, table, 0, 1, 1, 2);
-	gtk_table_attach_defaults(GTK_TABLE(table), pred->entries[basedir], 2, 3, 1, 2);
-
-	pred->entries[webdir] = entry_with_text(pred->project->webdir, 255);
-	bf_mnemonic_label_tad_with_alignment(_("_Preview URL:"), pred->entries[webdir], 1, 0.5, table, 0, 1, 2, 3);
-	gtk_table_attach_defaults(GTK_TABLE(table), pred->entries[webdir], 2, 3, 2, 3);
-	
-	pred->entries[template] = entry_with_text(pred->project->template, 255);
+/*	pred->entries[template] = entry_with_text(pred->project->template, 255);
 	bf_mnemonic_label_tad_with_alignment(_("_Template:"), pred->entries[template], 1, 0.5, table, 0, 1, 3, 4);
 	but = file_but_new(pred->entries[template], 1, NULL);
 	gtk_table_attach_defaults(GTK_TABLE(table), pred->entries[template], 2, 3, 3, 4);
-	gtk_table_attach_defaults(GTK_TABLE(table), but, 3, 4, 3, 4);
+	gtk_table_attach_defaults(GTK_TABLE(table), but, 3, 4, 3, 4);*/
+
+	sessionprefs(_("Initial document settings"),&pred->sprefs, pred->project->session);
+	gtk_box_pack_start(GTK_BOX(vbox), pred->sprefs.frame, FALSE, FALSE, 6);
 
-	gtk_table_set_row_spacing(GTK_TABLE(table), 3, 18);
-	pred->entries[word_wrap] = checkbut_with_value(_("_Word wrap by default"), pred->project->word_wrap);
-	gtk_table_attach_defaults(GTK_TABLE(table), pred->entries[word_wrap], 0, 3, 4, 5);
-		
-	gtk_box_pack_start (GTK_BOX (vbox), gtk_hseparator_new(), FALSE, FALSE, 12);
 	hbox = gtk_hbutton_box_new();
 	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
 	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 6);
@@ -616,7 +609,7 @@ static void project_create_ok_clicked_lcb(GtkWidget *widget, Tpc *pc) {
 void project_create_gui(Tbfwin *bfwin) {
 	GtkWidget *vbox, *hbox, *but;
 	Tpc *pc;
-	
+
 	pc = g_new(Tpc,1);
 	pc->bfwin = bfwin;
 	pc->win = window_full2(_("Create project"), GTK_WIN_POS_NONE, 5
@@ -637,7 +630,7 @@ void project_create_gui(Tbfwin *bfwin) {
 }
 
 void project_menu_cb(Tbfwin *bfwin,guint callback_action, GtkWidget *widget) {
-	DEBUG_MSG("project_menu_cb, bfwin=%p, callback_action=%d\n",bfwin,callback_action);
+	DEBUG_MSG("project_menu_cb, bfwin=%p, project=%p,callback_action=%d\n",bfwin,bfwin->project,callback_action);
 	switch (callback_action) {
 	case 1:
 		project_open(bfwin);
@@ -649,7 +642,32 @@ void project_menu_cb(Tbfwin *bfwin,guint callback_action, GtkWidget *widget) {
 		project_save(bfwin, TRUE);
 	break;
 	case 4:
-		project_save_and_close(bfwin);
+		/* if there are multiple windows we can simply close the window, else we have to convert it to a non-project window */
+		if (main_v->bfwinlist && main_v->bfwinlist->next) {
+			main_window_delete_event_lcb(NULL,NULL,bfwin);
+		} else {
+			project_save_and_mark_closed(bfwin);
+			if (!bfwin->documentlist) {
+				project_final_close(bfwin,FALSE);				
+			} else if (have_modified_documents(bfwin->documentlist)) {
+					Tclose_mode retval = multiple_files_modified_dialog(bfwin);
+				switch (retval) {
+				case close_mode_cancel:
+					return;
+				break;
+				case close_mode_per_file:
+					if (choose_per_file(bfwin, FALSE))
+						bfwin->project->close = TRUE;
+				break;
+				case close_mode_save_all:
+				case close_mode_close_all:
+					doc_close_multiple_backend(bfwin, FALSE, retval);
+				break;
+				}
+			} else {
+				doc_close_multiple_backend(bfwin, FALSE, close_mode_close_all);
+			}
+		}
 	break;
 	case 5:
 		project_edit(bfwin);
@@ -666,8 +684,9 @@ void project_menu_cb(Tbfwin *bfwin,guint callback_action, GtkWidget *widget) {
 		}
 	break;
 	default:
-		DEBUG_MSG("project_menu_cb, no such callback_action %d\n",callback_action);
-		exit(54);
+		DEBUG_MSG_C("uh-oh: project_menu_cb, no such callback_action %d\n",callback_action);
+		g_return_if_reached();
 	break;
 	}
 }
+
