@@ -15,15 +15,18 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
 #ifndef __PROJECT_H_
 #define __PROJECT_H_ 
 
 /* #define DEBUG */
-gboolean project_save_and_close(Tbfwin *bfwin);
-void project_open_from_file(Tbfwin *bfwin, gchar *fromfilename);
+gboolean project_save_and_close(Tbfwin *bfwin, gboolean close_win);
+void project_open_from_file(Tbfwin *bfwin, GFile *fromuri);
 void set_project_menu_widgets(Tbfwin *bfwin, gboolean win_has_project);
+void project_save_and_mark_closed(Tbfwin *bfwin);
+gboolean project_final_close(Tbfwin *bfwin, gboolean close_win);
 void project_menu_cb(Tbfwin *bfwin,guint callback_action, GtkWidget *widget);
 
 #endif /* __PROJECT_H_  */
