@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snr2.h - the search 'n replace rewrite prototypes
  *
- * Copyright (C) 2000-2002 Olivier Sessink
+ * Copyright (C) 2000-2009 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #ifndef __SNR2_H_
@@ -34,14 +34,26 @@ typedef struct {
 	they should be freed by the calling function! */
 	regmatch_t *pmatch;
 	gint nmatch;
+	Tdocument *doc; /* used when searching all documents */
+	gint errorcode; /* 0 means no error */
 } Tsearch_result;
 
-void snr2_init(Tbfwin *bfwin);
+/*typedef struct {
+	gint start;
+	gint end;
+	Tdocument *doc;
+} Tsearch_all_result;*/
 
-void snr2_run_extern_replace(Tdocument *doc, gchar *search_pattern, gint region,
-							gint matchtype, gint is_case_sens, gchar *replace_pattern,
-							gboolean store_as_last_snr2);
-Tsearch_result doc_search_run_extern(Tdocument *doc, gchar *search_pattern, gint matchtype, gint is_case_sens);
+typedef enum { match_normal, match_word, match_posix, match_perl } Tmatch_types;
+
+void snr2_init(Tbfwin *bfwin);
+void snr2_cleanup(Tbfwin *bfwin);
+
+gint snr2_run_extern_replace(Tdocument *doc, const gchar *search_pattern, gint region,
+							Tmatch_types matchtype, gint is_case_sens, const gchar *replace_pattern,
+							gboolean unescape);
+Tsearch_result doc_search_run_extern(Tdocument *doc, gchar *search_pattern, gint matchtype, gint is_case_sens, gint offset);
+void doc_show_result(Tdocument *document, GtkWindow *window, gint start, gint end, gboolean select_match);
 void search_from_selection(Tbfwin *bfwin);
 void search_cb(GtkWidget *widget, Tbfwin *bfwin);
 void replace_again_cb(GtkWidget *widget, Tbfwin *bfwin);
@@ -50,4 +62,16 @@ void replace_cb(GtkWidget *widget, Tbfwin *bfwin);
 void update_filenames_in_file(Tdocument *doc, gchar *oldfilename, gchar *newfilename, gint doc_has_newfilename);
 void update_encoding_meta_in_file(Tdocument *doc, gchar *encoding);
 
+enum {
+	BF_FIND_DIALOG = 0,
+	BF_REPLACE_DIALOG
+};
+void snr_dialog_new(Tbfwin * bfwin, gint dialogType);
+
+void strip_trailing_spaces(Tdocument *doc);
+void join_lines(Tdocument *doc);
+void split_lines(Tdocument *doc);
+void convert_identing(Tdocument *doc, gboolean to_tabs);
+void toggle_comment(Tdocument *doc);
+void convert_to_columns(Tdocument *doc);
 #endif							/* __SNR_H_ */
