@@ -15,9 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
-/*#define DEBUG*/
+
+/* #define DEBUG */
 
 /*
  * This module deals with the GUI dialog used in bluefish to manage
@@ -44,397 +45,17 @@
 #define STRING_MAX_SIZE 4096
 #define MAX_ARRAY_LENGTH 20
 
-
+/************************************************************************/
 /************************************************************************/
 
-
-/************************************************************************/
-#ifdef USE_ESTRL_DIALOG
-typedef struct {
-	GtkWidget *win;
-	GtkWidget *ins_upd_entry[MAX_ARRAY_LENGTH];
-	GtkWidget *clist;
-	gint num_columns;
-	gint last_row;
-	GList **the_list;
-	void (*function)();
-} Testrl;
-
-
-/******************************************************************************
- * Check if the strings we want to add, already exist in the list
- */
-static int is_duplicated(gchar **new_values, Testrl *estrl) {
-    guint i, duplicated=0;
-    gchar **tmparray, *tmpstring;
-    gint row=0, retval=1;
-
-    DEBUG_MSG("stringlist:is_duplicated(%p, %p)\n", new_values, estrl);
-    retval = gtk_clist_get_text(GTK_CLIST(estrl->clist), row, 0, &tmpstring);
-    while (retval == 1 && !duplicated) {
-      tmparray = g_malloc(((estrl->num_columns ? estrl->num_columns : 1)+2)*sizeof(char *));
-		for (i = 0; i < (estrl->num_columns ? estrl->num_columns  : 1); i++) {
-			gtk_clist_get_text(GTK_CLIST(estrl->clist), row, i, &tmpstring);
-			tmparray[i] = g_strdup(tmpstring);
-		}
-      tmparray[i] = NULL;
-
-      duplicated = 1;
-		if (estrl->num_columns == 0) {
-			if (strcmp((gchar *) tmparray[0], new_values[0]) != 0) {
-				duplicated = 0;
-			}
-		} else {
-			for (i = 0; i < estrl->num_columns; i++) {
-				DEBUG_MSG("old_values[%d]='%s'\n", i, tmparray[i]);
-				DEBUG_MSG("new_values[%d]='%s'\n", i, new_values[i]);
-				if (strcmp(tmparray[i], new_values[i]) != 0) {
-					duplicated = 0;
-				}
-			}
-		}
-      row++;
-      retval = gtk_clist_get_text(GTK_CLIST(estrl->clist), row, 0, &tmpstring);
-    }
-    DEBUG_MSG("duplicated=%d\n", duplicated);
-    return duplicated;
-}
-
-
-static void estrl_destroy_lcb(GtkWidget * widget, GdkEvent *event, Testrl *estrl) {
-	window_destroy(estrl->win);
-	g_free(estrl);
-}
-
-static void estrl_cancel_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
-	estrl_destroy_lcb(NULL, NULL, estrl);
-}
-
-/***************************************************************************
- * ok button clicked
- *
- * Creates a new list with the elements of the dialog and replaces the old list
- * with the new one
- * At the end calls the post_func passed to the creator of the dialog and
- * destroys the dialog
-*/
-static void estrl_ok_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
-
-	guint i;
-	gchar *tmpstring;
-	gchar **tmparray;
-	gint row=0, retval=1;
-	GList *the_list=NULL, *orig_list;
-
-	retval = gtk_clist_get_text(GTK_CLIST(estrl->clist), row, 0, &tmpstring);
-	while (retval == 1) {
-		tmparray = g_malloc(((estrl->num_columns ? estrl->num_columns : 1)+2)*sizeof(char *));
-		for (i = 0; i < (estrl->num_columns ? estrl->num_columns  : 1); i++) {
-			gtk_clist_get_text(GTK_CLIST(estrl->clist), row, i, &tmpstring);
-			tmparray[i] = g_strdup(tmpstring);
-		}
-		tmparray[i] = NULL;
-		if (estrl->num_columns == 0) {
-			the_list = g_list_append(the_list, tmparray[0]);
-			g_free(tmparray);
-		} else {
-			the_list = g_list_append(the_list, tmparray);
-		}
-		row++;
-		retval = gtk_clist_get_text(GTK_CLIST(estrl->clist), row, 0, &tmpstring);
-	}
-	DEBUG_MSG("estrl_ok_clicked_lcb, going to add %p to the list\n", the_list);
-	orig_list = *estrl->the_list;
-	*estrl->the_list = the_list;
-
-	if (estrl->num_columns) {
-	        free_arraylist(orig_list);
-	} else {
-		free_stringlist(orig_list);
-	}
-
-	if (estrl->function)
-		estrl->function();
-
-	estrl_destroy_lcb(NULL, NULL, estrl);
-}
-
-static void estrl_select_row_lcb(GtkWidget * widget, gint row, gint column, GdkEventButton * event, Testrl *estrl){
-	gchar *text;
-	guint i;
-
-	for (i = 0; i < ( estrl->num_columns ? estrl->num_columns : 1); i++) {
-		gtk_clist_get_text(GTK_CLIST(estrl->clist), row, i, &text);
-		gtk_entry_set_text(GTK_ENTRY(estrl->ins_upd_entry[i]), text);
-	}
-	estrl->last_row = row;
-}
-
-static void estrl_unselect_row_lcb(GtkWidget * widget, gint row, gint column, GdkEventButton * event, Testrl *estrl) {
-	estrl->last_row = -1;
-}
-
-static void estrl_up_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
-	if (estrl->last_row >= 1) {
-		gtk_clist_swap_rows(GTK_CLIST(estrl->clist), estrl->last_row, estrl->last_row - 1);
-		estrl->last_row--;
-	}
-}
-
-static void estrl_down_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
-	if (estrl->last_row < GTK_CLIST(estrl->clist)->rows - 1) {
-		gtk_clist_swap_rows(GTK_CLIST(estrl->clist), estrl->last_row, estrl->last_row + 1);
-		estrl->last_row++;
-	}
-}
-
-/******************************************************************************
- * Avoid adding an empty element to the stringlist
- * Avoid adding a duplicated element to the stringlist
- */
-static void estrl_add_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
-   gchar **tmplistentry;
-	guint i, len;
-
-    tmplistentry = g_malloc(((estrl->num_columns ? estrl->num_columns : 1)+2)*sizeof(char *));
-    for (len = i = 0; i < (estrl->num_columns ? estrl->num_columns  : 1); i++) {
-		tmplistentry[i] = gtk_editable_get_chars(GTK_EDITABLE(estrl->ins_upd_entry[i]),0,-1);
-		len += strlen(tmplistentry[i]);
-		DEBUG_MSG("tmplistentry[%d]='%s'\n", i, tmplistentry[i]);
-    }
-    tmplistentry[i]=NULL;
-    if(len) {
-		if(!is_duplicated(tmplistentry, estrl)) {
-			gtk_clist_append(GTK_CLIST(estrl->clist), tmplistentry);
-			/* does this introduce a memory leak?
-			I'm not going to look at it since we need to get rid of the clist anyway 
-			(gtk deprecated)
-			*/
-		} else {
-			g_free(tmplistentry);
-		}
-	} else {
-		g_free(tmplistentry);
-	}
-}
-
-static void estrl_update_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
-	guint i;
-
-	for (i = 0; i < (estrl->num_columns ? estrl->num_columns  : 1); i++) {
-		gtk_clist_set_text(GTK_CLIST(estrl->clist)
-					   , estrl->last_row, i, gtk_entry_get_text(GTK_ENTRY(estrl->ins_upd_entry[i])));
-	}
-}
-
-static void estrl_delete_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
-	gtk_clist_remove(GTK_CLIST(estrl->clist), estrl->last_row);
-}
-
-static void estrl_sort_clicked_lcb(GtkWidget * widget, Testrl *estrl) {
-	gtk_clist_set_auto_sort(GTK_CLIST(estrl->clist), 1);
-}
-
-/****************************************************************************
- * Creates the dialog to edit a stringlist
- *
- * Parameters:
- *  which_list      .- the stringlist which we want to edit
- *  title           .- the title of the dialog
- *  what_list       .- a bit more of information about the kind of strings
- *  column_num      .- number of strings in each element of the stringlist
- *                     equals to the number of columns in the dialog
- *                     if column_num=0, it is assumed column_num=1
- *  column_titles   .- title of the columns in the dialog
- *  post_dialog_func.- function to call after editing to process the modified
- *                     stringlist
-*/
-
-void estrl_dialog(GList **which_list, gchar *title, gint what_list
-				, gint column_num, gchar **column_titles, void (*post_dialog_func)()) {
-	Testrl *estrl;
-	GList *tmplist;
-	GtkWidget *but_var=NULL, *hbox, *scrolwin, *vbox2, *vbox;
-	GtkWidget *table2, *but;
-	gint i;
-	gchar *tmplistentry[1];
-
-
-	estrl = g_malloc(sizeof(Testrl));
-        if( column_num > MAX_ARRAY_LENGTH )
-              column_num = MAX_ARRAY_LENGTH;
-	estrl->num_columns = column_num;
-	estrl->the_list = which_list;
-	estrl->function = post_dialog_func;
-	estrl->win = window_full(title, GTK_WIN_POS_MOUSE, 5, G_CALLBACK(estrl_destroy_lcb), estrl, TRUE);
-
-	vbox = gtk_vbox_new(FALSE, 1);
-	gtk_container_add(GTK_CONTAINER(estrl->win), vbox);
-	gtk_widget_realize(estrl->win);
-
-	table2 = gtk_table_new(column_num ? column_num : 1, 3, 0);
-	for (i = 0; i < (column_num ? column_num : 1); i++) {
-		estrl->ins_upd_entry[i] = gtk_entry_new_with_max_length(950/(column_num ? column_num : 1));
-		if (column_titles) {
-			gtk_table_attach_defaults(GTK_TABLE(table2), gtk_label_new(column_titles[i]), 0, 1, i, i+1);
-		}
-		gtk_table_attach_defaults(GTK_TABLE(table2), estrl->ins_upd_entry[i], 1, 2, i, i+1);
-	}
-	gtk_box_pack_start(GTK_BOX(vbox), table2, FALSE, FALSE, 0);
-
-	switch (what_list) {
-	case 1:
-		/* files */
-		but_var = file_but_new(estrl->ins_upd_entry[0], estrl->win, 1);
-		break;
-	case 2:
-		/* color */
-#ifdef NOTYET
-		but_var = color_but_new(estrl->ins_upd_entry[0], estrl->win);
-#endif
-		break;
-	case 4:
-		/* url */
-		but_var = file_but_new(estrl->ins_upd_entry[0], estrl->win, 0);
-		break;
-	case 7:
-		/* external filters and commands */
-
-		break;
-	case 8:
-		/* syntax highlighting patterns */
-#ifdef NOTYET
-		but_var = color_but_new(estrl->ins_upd_entry[0], estrl->win);
-#endif
-		break;
-
-#ifdef PARSEDTD
-	case 9:
-                /* doctypes */
-		but_var = file_but_new(estrl->ins_upd_entry[1], estrl->win, 0);
-		break;
-#endif
-
-	default:
-		/* targets(5), fonts(3) meta_cb (6) */
-		break;
-	}
-	if (but_var) {
-		gtk_table_attach_defaults(GTK_TABLE(table2), but_var, 2, 3, 0, 1);
-		GTK_WIDGET_SET_FLAGS(but_var, GTK_CAN_DEFAULT);
-	}
-
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-	scrolwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(hbox), scrolwin, TRUE, TRUE, 0);
-	vbox2 = gtk_vbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 0);
-
-	if (column_num == 0) {
-		estrl->clist = gtk_clist_new(1);
-	} else {
-		if (column_titles != NULL) {
-			estrl->clist = gtk_clist_new_with_titles(column_num, column_titles);
-		} else {
-			estrl->clist = gtk_clist_new(column_num);
-		}
-	}
-
-	gtk_signal_connect(GTK_OBJECT(estrl->clist), "select_row", GTK_SIGNAL_FUNC(estrl_select_row_lcb), estrl);
-	gtk_signal_connect(GTK_OBJECT(estrl->clist), "unselect_row", GTK_SIGNAL_FUNC(estrl_unselect_row_lcb), estrl);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolwin), estrl->clist);
-	gtk_clist_set_reorderable(GTK_CLIST(estrl->clist), TRUE);
-	gtk_clist_freeze(GTK_CLIST(estrl->clist));
-
-	tmplist = g_list_first(*which_list);
-	if (column_num == 0) {
-		while (tmplist != NULL) {
-			tmplistentry[0] = (gchar *) tmplist->data;
-			gtk_clist_append(GTK_CLIST(estrl->clist), tmplistentry);
-			tmplist = g_list_next(tmplist);
-		}
-	} else {
-		while (tmplist != NULL) {
-			gtk_clist_append(GTK_CLIST(estrl->clist), (gchar **) tmplist->data);
-			tmplist = g_list_next(tmplist);
-		}
-	}
-	gtk_clist_thaw(GTK_CLIST(estrl->clist));
-
-	but = bf_generic_mnemonic_button(_(" Add "), G_CALLBACK(estrl_add_clicked_lcb), estrl);
-	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_generic_mnemonic_button(_(" Update "), G_CALLBACK(estrl_update_clicked_lcb), estrl);
-	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_generic_mnemonic_button(_(" Delete "), G_CALLBACK(estrl_delete_clicked_lcb), estrl);
-	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_generic_mnemonic_button(_(" Up "), G_CALLBACK(estrl_up_clicked_lcb), estrl);
-	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_generic_mnemonic_button(_(" Down "), G_CALLBACK(estrl_down_clicked_lcb), estrl);
-	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_generic_mnemonic_button(_(" Sort "), G_CALLBACK(estrl_sort_clicked_lcb), estrl);
-	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-
-	hbox = gtk_hbutton_box_new();
-	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 1);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-
-	but = bf_stock_ok_button(G_CALLBACK(estrl_ok_clicked_lcb), estrl);
-	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
-	gtk_window_set_default(GTK_WINDOW(estrl->win), but);
-
-	but = bf_stock_cancel_button(G_CALLBACK(estrl_cancel_clicked_lcb), estrl);
-	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
-
-	/* Adjust the size and position of the dialog */
-	{
-	  int wx=100, optx;
-	  optx = gtk_clist_optimal_column_width(GTK_CLIST(estrl->clist),0);
-	  if( optx < 100 )
-	    optx = 100;
-	  wx += optx + gtk_clist_optimal_column_width(GTK_CLIST(estrl->clist), 1);
-	  if( wx < gdk_screen_width() ) {
-	    gtk_clist_set_column_width(GTK_CLIST(estrl->clist), 0, optx );
-	    gtk_widget_set_size_request(GTK_WIDGET(estrl->win), wx, 400);
-	    gtk_widget_set_uposition(estrl->win, 
-				     (gdk_screen_width()-wx)/2,
-				     (gdk_screen_height() - 400)/2);
-	  }
-	}
-	gtk_widget_show_all(estrl->win);
-}
-#endif /* USE_ESTRL_DIALOG */
-
-
-/************************************************************************/
-
-#ifdef DEBUG
-void debug_array(gchar **array) {
-	gint count=0;
-	gchar **tmpchar=array;
-	
-	if (!tmpchar) {
-		DEBUG_MSG("debug_array, no array!?!?\n");
-	}
-
-	while (*tmpchar != NULL) {
-		count++;
-		DEBUG_MSG("debug_array, tmpchar(%p), count=%d, contains(%p) %s\n", tmpchar, count, *tmpchar, *tmpchar);
-		tmpchar++;
-	}
-}
-#endif
-
-/**
+/* DEPRECATED BY g_strv_length
  * count_array:
  * @array: #gchar** with the NULL terminated array to count
  *
  * counts the number of entries in a NULL terminated array
  *
  * Return value: #gint with number of entries
- */
+ * /
 gint count_array(gchar **array) {
 	gint count=0;
 	gchar **tmpchar=array;
@@ -450,7 +71,7 @@ gint count_array(gchar **array) {
 	}
 	return count;
 }
-
+*/
 /**
  * array_to_string:
  * @array: #gchar** with NULL terminated array
@@ -464,33 +85,25 @@ gint count_array(gchar **array) {
  * Return value: #gchar* newly allocated string
  */
 gchar *array_to_string(gchar **array) {
-	if (array) {
-		gchar **tmp, *escaped1, *finalstring;
-		gint newsize=1;
-		DEBUG_MSG("array_to_string, started\n");
-		finalstring = g_malloc0(newsize);
-		tmp = array;
-		while(*tmp) {
-			DEBUG_MSG("array_to_string, *tmp = %s\n", *tmp);
-			escaped1 = escape_string(*tmp, TRUE);
-			newsize += strlen(escaped1)+1;
-			finalstring = g_realloc(finalstring, newsize);
-			strcat(finalstring, escaped1);
-			finalstring[newsize-2] = ':';
-			finalstring[newsize-1] = '\0';
-			g_free(escaped1);
-			tmp++;
-		}	
-		DEBUG_MSG("array_to_string, finalstring = %s\n", finalstring);
-		return finalstring;
-	} else {
-#ifdef DEBUG
-		g_print("array_to_string, array=NULL !!!\n");
-		exit(135);
-#else
-		return g_strdup("");
-#endif
-	}
+	gchar **tmp, *escaped1, *finalstring;
+	gint newsize=1;
+	g_return_val_if_fail(array,g_strdup(""));
+	DEBUG_MSG("array_to_string, started\n");
+	finalstring = g_malloc0(newsize);
+	tmp = array;
+	while(*tmp) {
+		DEBUG_MSG("array_to_string, *tmp = %s\n", *tmp);
+		escaped1 = escape_string(*tmp, TRUE);
+		newsize += strlen(escaped1)+1;
+		finalstring = g_realloc(finalstring, newsize);
+		strcat(finalstring, escaped1);
+		finalstring[newsize-2] = ':';
+		finalstring[newsize-1] = '\0';
+		g_free(escaped1);
+		tmp++;
+	}	
+	DEBUG_MSG("array_to_string, finalstring = %s\n", finalstring);
+	return finalstring;
 }
 
 #define ARRAYBLOCKSIZE 6
@@ -559,9 +172,9 @@ gchar **string_to_array(gchar *string) {
 			arraycount++;
 			if (arraycount == arraysize-2) { /* we need 1 spare entry in the array */
 				arraysize += ARRAYBLOCKSIZE;  /* and arraysize starts at 1, arraycount at 0 */
-				DEBUG_MSG("string_to_array, arraycount=%d, about to increase arraysize to %d, sizeof(array(%p))=%d\n", arraycount, arraysize, array, sizeof(&array));
+				DEBUG_MSG("string_to_array, arraycount=%d, about to increase arraysize to %d, sizeof(array(%p))=%ld\n", arraycount, arraysize, array, sizeof(&array));
 				array = g_realloc(array, arraysize * sizeof(char *));
-				DEBUG_MSG("string_to_array, arraysize=%d, array(%p), sizeof(array)=%d\n", arraysize, array, sizeof(&array));
+				DEBUG_MSG("string_to_array, arraysize=%d, array(%p), sizeof(array)=%ld\n", arraysize, array, sizeof(&array));
 			}
 			count = -1;
 		} else {
@@ -573,7 +186,7 @@ gchar **string_to_array(gchar *string) {
 			newstringsize += BUFBLOCKSIZE;
 			DEBUG_MSG("string_to_array, about to increase newstring(%p) to %d bytes\n", newstring, newstringsize);
 			newstring = g_realloc(newstring, newstringsize * sizeof(char));
-			DEBUG_MSG("string_to_array, newstringsize=%d, sizeof(newstring(%p))=%d\n", newstringsize, newstring, sizeof(newstring));
+			DEBUG_MSG("string_to_array, newstringsize=%d, sizeof(newstring(%p))=%ld\n", newstringsize, newstring, sizeof(newstring));
 		}
 	}
 	
@@ -722,7 +335,7 @@ gint free_arraylist(GList * which_list)
 	which_list = NULL;
 	return 1;
 }
-
+/* deprecated by g_strdupv
 gchar **duplicate_stringarray(gchar **array) {
 	gchar **newchar;
 	gint i;
@@ -732,7 +345,7 @@ gchar **duplicate_stringarray(gchar **array) {
 		newchar[i] = g_strdup(array[i]);
 	}
 	return newchar;
-}
+}*/
 
 GList *duplicate_arraylist(GList *arraylist) {
 	GList *tmplist;
@@ -746,46 +359,90 @@ GList *duplicate_arraylist(GList *arraylist) {
 	return newlist;
 }
 
+GList *get_list_from_buffer(gchar *buffer, GList *which_list, gboolean is_arraylist) {
+	gchar *pos,*nextpos;
+	pos = buffer;
+	nextpos = strchr(pos, '\n');
+	do {
+		if (nextpos)
+			*nextpos='\0';
+		/*g_print("next line: %s\n",pos);*/
+		if (is_arraylist) {
+			gchar **temparr = string_to_array(pos);
+			which_list = g_list_append(which_list, temparr);
+		} else {
+			DEBUG_MSG("get_list, adding string \"%s\" to the stringlist=%p\n", pos, which_list);
+			which_list = g_list_append(which_list, g_strdup(pos));
+		}
+		if (nextpos) {
+			nextpos++;
+			if (*nextpos == '\0')
+				pos = NULL;
+			else
+				pos = nextpos;
+		} else 
+			pos = NULL;
+		if (pos)
+			nextpos = strchr(pos, '\n');
+	} while (pos);
+	
+	return which_list;
+}
 
 /*****************************************************************************
  * gets a stringlist from a file
  */
-GList *get_list(const gchar * filename, GList * which_list, gboolean is_arraylist) {
+GList *get_list(GFile * file, GList * which_list, gboolean is_arraylist) {
+	gchar *buffer=NULL;
+	gsize length;
+	GError *error=NULL;
+	if (file == NULL) {
+		return NULL;
+	}
+	g_file_load_contents(file,NULL,&buffer,&length,NULL,&error);
+	if (error) {
+		g_print("error reading list %d %s\n",error->code, error->message);
+		return NULL;
+	}
+	which_list = get_list_from_buffer(buffer, which_list, is_arraylist);
+	g_free(buffer);
+	
+/*	FILE *fd;
+	gchar *tmpbuf, *tmpfilename;
+
 	DEBUG_MSG("get_stringlist, started with filename=%s\n", filename);
 	if (filename == NULL) {
 		return NULL;
 	}
-	if (file_exists_and_readable(filename)) {
-		FILE *fd;
-		gchar *tmpbuf;
-		DEBUG_MSG("get_stringlist, opening %s\n", filename);
-		fd = fopen(filename, "r");
-		if (fd == NULL) {
-			return NULL;
-		}
-		tmpbuf = g_malloc(STRING_MAX_SIZE);
-		while (fgets(tmpbuf, STRING_MAX_SIZE, fd) != NULL) {
-			gchar *tempstr;
-			tmpbuf = trunc_on_char(tmpbuf, '\n');
-			tempstr = g_strdup(tmpbuf);
-			if (is_arraylist) {
-				gchar **temparr = string_to_array(tempstr);
-				which_list = g_list_append(which_list, temparr);
-				g_free(tempstr);
-			} else {
-				DEBUG_MSG("get_list, adding string \"%s\" to the stringlist=%p\n", tempstr, which_list);
-				which_list = g_list_append(which_list, tempstr);
-			}
-		}
-		fclose(fd);
-		g_free(tmpbuf);
+	tmpfilename = g_strdup(filename+((strncmp(filename, "file://",7)==0)?7:0));
+	DEBUG_MSG("get_stringlist, opening %s\n", tmpfilename);
+	fd = fopen(tmpfilename, "r");
+	g_free(tmpfilename);
+	if (fd == NULL) {
+		return NULL;
 	}
+	tmpbuf = g_malloc(STRING_MAX_SIZE);
+	while (fgets(tmpbuf, STRING_MAX_SIZE, fd) != NULL) {
+		gchar *tempstr;
+		tmpbuf = trunc_on_char(tmpbuf, '\n');
+		tempstr = g_strdup(tmpbuf);
+		if (is_arraylist) {
+			gchar **temparr = string_to_array(tempstr);
+			which_list = g_list_append(which_list, temparr);
+			g_free(tempstr);
+		} else {
+			DEBUG_MSG("get_list, adding string \"%s\" to the stringlist=%p\n", tempstr, which_list);
+			which_list = g_list_append(which_list, tempstr);
+		}
+	}
+	fclose(fd);
+	g_free(tmpbuf);*/
 	return which_list;
 }
 #ifdef __GNUC__
 __inline__ 
 #endif
-GList *get_stringlist(const gchar * filename, GList * which_list) {
+GList *get_stringlist(GFile * filename, GList * which_list) {
 	return get_list(filename,which_list,FALSE);
 }
 
@@ -799,23 +456,12 @@ GList *get_stringlist(const gchar * filename, GList * which_list) {
  * if maxentries is 0 all entries will be stored
  *
  * Return value: #gboolean TRUE on success, FALSE on failure
- */
-gboolean put_stringlist_limited(gchar * filename, GList * which_list, gint maxentries) {
-	FILE *fd;
+ * /
+gboolean put_stringlist_limited(GFile * file, GList * which_list, gint maxentries, gboolean is_arraylist) {
+	GString *strbuffer;
+	GError *error=NULL;
 	GList *tmplist;
-
-	DEBUG_MSG("put_stringlist_limited, started with filename=%s\n", filename);
-	{
-		gchar *backupfilename = g_strconcat(filename, main_v->props.backup_filestring,NULL);
-		file_copy(filename, backupfilename);
-		g_free(backupfilename);
-	}
-
-	DEBUG_MSG("put_stringlist_limited, opening %s for saving list(%p)\n", filename, which_list);
-	fd = fopen(filename, "w");
-	if (fd == NULL) {
-		return FALSE;
-	}
+	DEBUG_MSG("put_stringlist_limited, limit file %s to %d entries\n", g_file_get_uri(file), maxentries);
 	if (maxentries > 0) {
 		gint count;
 		count = g_list_length(which_list) - maxentries;
@@ -823,21 +469,65 @@ gboolean put_stringlist_limited(gchar * filename, GList * which_list, gint maxen
 	} else {
 		tmplist = g_list_first(which_list);
 	}
-	while (tmplist) {
-		gchar *tmpstr = g_strndup((char *) tmplist->data, STRING_MAX_SIZE - 1);
-		DEBUG_MSG("put_stringlist_limited, tmplist(%p), adding string(%p)=%s (strlen=%d)the file\n", tmplist, tmpstr, tmpstr, strlen(tmpstr));
-		fputs(tmpstr, fd);
-		g_free(tmpstr);
-		fputs("\n", fd);
-		tmplist = g_list_next(tmplist);
+	strbuffer = g_string_sized_new(1024);
+	if (is_arraylist) {
+		while (tmplist) {
+			gchar *tmp = array_to_string(tmplist->data);
+			strbuffer = g_string_append(strbuffer,tmp);
+			g_free(tmp);
+			strbuffer = g_string_append_c(strbuffer,'\n');
+			tmplist = g_list_next(tmplist);
+		}
+	} else {
+		while (tmplist) {
+			strbuffer = g_string_append(strbuffer,(char *) tmplist->data);
+			strbuffer = g_string_append_c(strbuffer,'\n');
+			tmplist = g_list_next(tmplist);
+		}
 	}
-	fclose(fd);
+	g_file_replace_contents(file,strbuffer->str,strbuffer->len
+				,NULL,FALSE,G_FILE_CREATE_PRIVATE,NULL,NULL,&error);
+	if (error) {
+		g_warning("save stringlist error %d %s\n",error->code,error->message);
+		g_error_free(error);
+		return FALSE;
+	}
+	g_string_free(strbuffer,TRUE);
 	DEBUG_MSG("put_stringlist_limited, finished, filedescriptor closed\n");
 	return TRUE;
-}
+}*/
 
-gboolean put_stringlist(gchar * filename, GList * which_list) {
-	return put_stringlist_limited(filename,which_list, -1);
+gboolean put_stringlist(GFile * file, GList * which_list, gboolean is_arraylist) {
+	/*return put_stringlist_limited(file,which_list, -1, is_arraylist);*/
+	GString *strbuffer;
+	GError *gerror=NULL;
+	GList *tmplist = g_list_first(which_list);;
+	
+	strbuffer = g_string_sized_new(1024);
+	if (is_arraylist) {
+		while (tmplist) {
+			gchar *tmp = array_to_string(tmplist->data);
+			strbuffer = g_string_append(strbuffer,tmp);
+			g_free(tmp);
+			strbuffer = g_string_append_c(strbuffer,'\n');
+			tmplist = g_list_next(tmplist);
+		}
+	} else {
+		while (tmplist) {
+			strbuffer = g_string_append(strbuffer,(char *) tmplist->data);
+			strbuffer = g_string_append_c(strbuffer,'\n');
+			tmplist = g_list_next(tmplist);
+		}
+	}
+	g_file_replace_contents(file,strbuffer->str,strbuffer->len
+				,NULL,FALSE,G_FILE_CREATE_PRIVATE,NULL,NULL,&gerror);
+	if (gerror) {
+		g_warning("save stringlist error %d %s\n",gerror->code,gerror->message);
+		g_error_free(gerror);
+		return FALSE;
+	}
+	g_string_free(strbuffer,TRUE);
+	return TRUE;
 }
 
 GList *remove_from_stringlist(GList *which_list, const gchar * string) {
@@ -857,6 +547,7 @@ GList *remove_from_stringlist(GList *which_list, const gchar * string) {
 
 static void unlink_before(GList *tmplist) {
 	GList *prev = tmplist->prev;
+	DEBUG_MSG("unlink_before, freeing starting at %s\n",(gchar *)tmplist->data);
 	if (prev) {
 		prev->next = NULL;
 	}
@@ -977,6 +668,7 @@ gchar *stringlist_to_string(GList *stringlist, gchar *delimiter) {
 	}
 	return string;
 }
+
 /**
  * array_n_strings_identical:
  * @array1: #gchar**
@@ -993,7 +685,7 @@ gchar *stringlist_to_string(GList *stringlist, gchar *delimiter) {
  *
  * Return value: #gint
  */
-gint array_n_strings_identical(gchar **array1, gchar **array2, gboolean case_sensitive, gint testlevel) {
+gint array_n_strings_identical(const gchar **array1, const gchar **array2, gboolean case_sensitive, gint testlevel) {
 	gint i=0, res=0;
 	while (i<testlevel && res==0) {
 		/*  array1[i]==array2[i] will only happen when they are both NULL	*/
@@ -1023,12 +715,12 @@ gint array_n_strings_identical(gchar **array1, gchar **array2, gboolean case_sen
  *
  * Return value: the new list
  */
-GList *arraylist_delete_identical(GList *thelist, gchar **compare, gint testlevel, gboolean case_sensitive) {
+GList *arraylist_delete_identical(GList *thelist, const gchar **compare, gint testlevel, gboolean case_sensitive) {
 	GList *tmplist = g_list_first(thelist);
 	while (tmplist) {
 		GList *nextlist = g_list_next(tmplist);
 		gchar **tmparr = tmplist->data;
-		if (array_n_strings_identical(compare, tmparr, case_sensitive, testlevel)==0) {
+		if (array_n_strings_identical(compare, (const gchar **)tmparr, case_sensitive, testlevel)==0) {
 			DEBUG_MSG("arraylist_delete_identical, %s and %s are identical, will delete %p from list\n",tmparr[0],compare[0], tmplist);
 			thelist = g_list_delete_link(thelist, tmplist);
 			DEBUG_MSG("arraylist_delete_identical, free array %p (%s)\n",tmparr,tmparr[0]);
@@ -1052,11 +744,11 @@ GList *arraylist_delete_identical(GList *thelist, gchar **compare, gint testleve
  *
  * Return value: the new arraylist
  */
-GList *arraylist_append_identical_from_list(GList *thelist, GList *source, gchar **compare, gint testlevel, gboolean case_sensitive) {
+GList *arraylist_append_identical_from_list(GList *thelist, GList *source, const gchar **compare, gint testlevel, gboolean case_sensitive) {
 	GList *tmplist = g_list_first(source);
 	while (tmplist) {
 		gchar **tmparr = tmplist->data;
-		if (array_n_strings_identical(compare, tmparr, case_sensitive, testlevel)==0) {
+		if (array_n_strings_identical(compare, (const gchar **)tmparr, case_sensitive, testlevel)==0) {
 			thelist = g_list_append(thelist, duplicate_stringarray(tmparr));
 		}
 		tmplist = g_list_next(tmplist);
@@ -1076,13 +768,13 @@ GList *arraylist_append_identical_from_list(GList *thelist, GList *source, gchar
  * add the array to 'thelist'
  *
  * Return value: the new arraylist
- */
-GList *arraylist_append_identical_from_file(GList *thelist, const gchar *sourcefilename, gchar **compare, gint testlevel, gboolean case_sensitive) {
+ * /
+GList *arraylist_append_identical_from_file(GList *thelist, const gchar *sourcefilename, const gchar **compare, gint testlevel, gboolean case_sensitive) {
 	GList *sourcelist = get_list(sourcefilename,NULL,TRUE);
 	thelist = arraylist_append_identical_from_list(thelist, sourcelist, compare, testlevel, case_sensitive);
 	free_arraylist(sourcelist);
 	return thelist;
-}
+}*/
 
 /**
  * arraylist_value_exists:
@@ -1096,18 +788,20 @@ GList *arraylist_append_identical_from_file(GList *thelist, const gchar *sourcef
  * so if you want the test to check for a 100% identical array that number 
  * should be high (9999 or so)
  *
- * Return value: #gboolean 
+ * Return value: #gchar ** 
  */
-gboolean arraylist_value_exists(GList *arraylist, gchar **value, gint testlevel, gboolean case_sensitive) {
+gchar **arraylist_value_exists(GList *arraylist, const gchar **value, gint testlevel, gboolean case_sensitive) {
 	GList *tmplist = g_list_first(arraylist);
 	while (tmplist) {
 		gchar **tmparr = tmplist->data;
-		if (array_n_strings_identical(value, tmparr, case_sensitive, testlevel)==0) {
-			return TRUE;
+		if (array_n_strings_identical(value, (const gchar **)tmparr, case_sensitive, testlevel)==0) {
+/*			g_print("arraylist_value_exists,found it!\n");*/
+			return tmparr;
 		}
 		tmplist = g_list_next(tmplist);
 	}
-	return FALSE;
+/*	g_print("arraylist_value_exists,found nothing, listlen=%d\n",g_list_length(arraylist));*/
+	return NULL;
 }
 /**
  * arraylist_load_new_identifiers_from_list:
@@ -1128,7 +822,7 @@ GList *arraylist_load_new_identifiers_from_list(GList *mylist, GList *deflist, g
 	while (tmplist) {
 		gchar **tmparr = tmplist->data;
 		if (count_array(tmparr) >= uniquelevel) {
-			if (!arraylist_value_exists(mylist, tmparr, uniquelevel, TRUE)) {
+			if (!arraylist_value_exists(mylist, (const gchar **)tmparr, uniquelevel, TRUE)) {
 				DEBUG_MSG("arraylist_load_new_identifiers, adding %s to thelist\n",tmparr[0]);
 				mylist = g_list_append(mylist, duplicate_stringarray(tmparr));
 			}
@@ -1150,10 +844,24 @@ GList *arraylist_load_new_identifiers_from_list(GList *mylist, GList *deflist, g
  * at the end of the function
  *
  * Return value: #GList*
- */
+ * / 
 GList *arraylist_load_new_identifiers_from_file(GList *mylist, const gchar *fromfilename, gint uniquelevel) {
 	GList *deflist = get_list(fromfilename,NULL,TRUE);
 	mylist = arraylist_load_new_identifiers_from_list(mylist, deflist, uniquelevel);
 	free_arraylist(deflist);
 	return mylist;	
+}*/
+
+/* pure for debugging purposes */
+#ifdef DEVELOPMENT
+void dump_arraylist(GList *list) {
+	GList *tmplist;
+	for (tmplist=g_list_first(list);tmplist;tmplist=tmplist->next) {
+		gchar *tmp = array_to_string((gchar **)tmplist->data);
+		g_print("%s\n",tmp);
+		g_free(tmp);
+	}
 }
+#endif
+
+
